#ifndef _CF_H
#define _CF_H

#include <sys/bsdtypes.h>


#include <cyg/hal/io.h>


// IDE Register Indices
#define IDE_REG_DATA      0
#define IDE_REG_ERROR     1
#define IDE_REG_FEATURES  1
#define IDE_REG_COUNT     2
#define IDE_REG_REASON    2  // ATAPI
#define IDE_REG_LBALOW    3
#define IDE_REG_LBAMID    4
#define IDE_REG_LBAHI     5
#define IDE_REG_DEVICE    6
#define IDE_REG_STATUS    7
#define IDE_REG_COMMAND   7

#define IDE_STAT_BSY      0x80
#define IDE_STAT_DRDY     0x40
#define IDE_STAT_SERVICE  0x10
#define IDE_STAT_DRQ      0x08
#define IDE_STAT_CORR     0x04
#define IDE_STAT_ERR      0x01

#define IDE_REASON_REL    0x04
#define IDE_REASON_IO     0x02
#define IDE_REASON_COD    0x01

/* flag values */

#define IDE_DEV_PRESENT  1  // Device is present
#define IDE_DEV_PACKET   2  // Supports packet interface
#define IDE_DEV_ADDR48   3  // Supports 48bit addressing

#include <cyg/infra/cyg_type.h>

// Initialize the IDE controller(s).

#define ALT_AVALON_CF_READ_UINT8( __ctlr, __regno, __val )  \
  HAL_READ_UINT8 (((__ctlr) + (__regno)), __val)

#define ALT_AVALON_CF_READ_UINT16( __ctlr, __regno, __val )  \
  HAL_READ_UINT16 (((__ctlr) + (__regno)), __val)

#define ALT_AVALON_CF_READ_ALTSTATUS( __ctlr, __val )  \
  HAL_READ_UINT32 (((__ctlr) + 14), __val)

#define ALT_AVALON_CF_WRITE_UINT8( __ctlr, __regno, __val )  \
  HAL_WRITE_UINT8(((__ctlr) + (__regno)), __val)

#define ALT_AVALON_CF_WRITE_UINT16( __ctlr, __regno, __val )  \
  HAL_WRITE_UINT16(((__ctlr) + (__regno)), __val)

#define ALT_AVALON_CF_WRITE_CONTROL( __ctlr, __val )  \
  HAL_WRITE_UINT16(((__ctlr) + 14), __val)


/* Alternate status register... */
typedef enum
{
  AltStatusRegBitERR  = 1,
  AltStatusRegBitIDX  = 2,
  AltStatusRegBitCORR = 4,
  AltStatusRegBitDRQ  = 8,
  AltStatusRegBitDSC  = 0x10,
  AltStatusRegBitDF   = 0x20,
  AltStatusRegBitDRDY = 0x40,
  AltStatusRegBitBSY  = 0x80
} IDE_ALTERNATE_STATUS_REGISTER_BITS;

/* Device control register... */
typedef enum
{
  DevControlRegBitNIEN = 2,
  DevControlRegBitSRST = 4,
} IDE_DEVICE_CONTROL_REGISTER_BITS;

/* Device/Head register... */
typedef enum
{
  DevHeadRegBitHeads   = 0xF,
  DevHeadRegBitDEV     = 0x10,
  DevHeadRegBitLBAMode = 0x40,
} IDE_DEVICE_HEAD_REGISTER_BITS;

/* Error register... */
typedef enum
{
  ErrorRegBitAMNF = 1,
  ErrorRegBitTKNONF = 2,
  ErrorRegBitABRT = 4,
  ErrorRegBitMCR = 8,
  ErrorRegBitIDNF = 0x10,
  ErrorRegBitMC = 0x20,
  ErrorRegBitUNC = 0x40,
} IDE_ERROR_REGISTER_BITS;

/* Status register... */
typedef enum
{
  StatusRegBitERR  = 1,
  StatusRegBitIDX  = 2,
  StatusRegBitCORR = 4,
  StatusRegBitDRQ  = 8,
  StatusRegBitDSC  = 0x10,
  StatusRegBitDF   = 0x20,
  StatusRegBitDRDY = 0x40,
  StatusRegBitBSY  = 0x80
} IDE_STATUS_REGISTER_BITS;

/* IDE command definitions */
typedef enum
{
  IdentifyDeviceCmd             = 0xEC,
  ReadSectorWithRetries         = 0x20,
  ReadSectorWithoutRetries      = 0x21,
  WriteSectorWithRetries        = 0x30,
  WriteSectorWithoutRetries     = 0x31,
  InitializeDeviceParameters    = 0x91,
  ExecuteDeviceDiagnostic       = 0x90,
  DMAReadSectorWithRetries      = 0xC8,
  DMAReadSectorWithoutRetries   = 0xC9,
  DMAWriteSectorWithRetries     = 0xCA,
  DMAWriteSectorWithoutRetries  = 0xCB,
} IDE_COMMANDS;


typedef struct ide_identify_data_t_ {
    cyg_uint16 general_conf;         // 00    : general configuration
    cyg_uint16 num_cylinders;        // 01    : number of cylinders (default CHS trans)
    cyg_uint16 reserved0;            // 02    : reserved
    cyg_uint16 num_heads;            // 03    : number of heads (default CHS trans)
    cyg_uint16 num_ub_per_track;     // 04    : number of unformatted bytes per track
    cyg_uint16 num_ub_per_sector;    // 05    : number of unformatted bytes per sector
    cyg_uint16 num_sectors;          // 06    : number of sectors per track (default CHS trans)
    cyg_uint16 num_card_sectors[2];  // 07-08 : number of sectors per card
    cyg_uint16 reserved1;            // 09    : reserved
    cyg_uint16 serial[10];           // 10-19 : serial number (string)
    cyg_uint16 buffer_type;          // 20    : buffer type (dual ported)
    cyg_uint16 buffer_size;          // 21    : buffer size in 512 increments
    cyg_uint16 num_ECC_bytes;        // 22    : number of ECC bytes passed on R/W Long cmds
    cyg_uint16 firmware_rev[4];      // 23-26 : firmware revision (string)
    cyg_uint16 model_num[20];        // 27-46 : model number (string)
    cyg_uint16 rw_mult_support;      // 47    : max number of sectors on R/W multiple cmds
    cyg_uint16 reserved2;            // 48    : reserved
    cyg_uint16 capabilities;         // 49    : LBA, DMA, IORDY support indicator
    cyg_uint16 reserved3;            // 50    : reserved
    cyg_uint16 pio_xferc_imting;     // 51    : PIO data transfer cycle timing mode
    cyg_uint16 dma_xferc_timing;     // 52    : single word DMA data transfer cycle timing mode
    cyg_uint16 cur_field_validity;   // 53    : words 54-58 validity (0 == not valid)
    cyg_uint16 cur_cylinders;        // 54    : number of current cylinders
    cyg_uint16 cur_heads;            // 55    : number of current heads
    cyg_uint16 cur_spt;              // 56    : number of current sectors per track
    cyg_uint16 cur_capacity[2];      // 57-58 : current capacity in sectors
    cyg_uint16 mult_sectors;         // 59    : multiple sector setting
    cyg_uint16 lba_total_sectors[2]; // 60-61 : total sectors in LBA mode
    cyg_uint16 sw_dma;               // 62    : single word DMA support
    cyg_uint16 mw_dma;               // 63    : multi word DMA support
    cyg_uint16 apio_modes;           // 64    : advanced PIO transfer mode supported
    cyg_uint16 min_dma_timing;       // 65    : minimum multiword DMA transfer cycle
    cyg_uint16 rec_dma_timing;       // 66    : recommended multiword DMA cycle
    cyg_uint16 min_pio_timing;       // 67    : min PIO transfer time without flow control
    cyg_uint16 min_pio_iordy_timing; // 68    : min PIO transfer time with IORDY flow control
//  cyg_uint16 reserved4[187];       // 69-255: reserved
} ide_identify_data_t;

struct alt_avalon_cf_interface;

typedef struct {
  cyg_uint32* ide_base;
  cyg_uint8   chan;
  struct alt_avalon_cf_interface* interface;
} alt_avalon_cf_info_t;


/* IDE Slave I/O RD/WR/ADDR macros */
#define IOADDR_ALTERA_AVALON_CF_IDE_DATA(base)               __IO_CALC_ADDRESS_NATIVE(base, 0)
#define IORD_ALTERA_AVALON_CF_IDE_DATA(base)                 IORD(base, 0)
#define IOWR_ALTERA_AVALON_CF_IDE_DATA(base, data)           IOWR(base, 0, data)

#define IOADDR_ALTERA_AVALON_CF_IDE_ERROR(base)              __IO_CALC_ADDRESS_NATIVE(base, 1)
#define IORD_ALTERA_AVALON_CF_IDE_ERROR(base)                IORD(base, 1)
#define IOWR_ALTERA_AVALON_CF_IDE_FEATURES(base, data)       IOWR(base, 1, data)

#define IOADDR_ALTERA_AVALON_CF_IDE_SECTOR_COUNT(base)       __IO_CALC_ADDRESS_NATIVE(base, 2)
#define IORD_ALTERA_AVALON_CF_IDE_SECTOR_COUNT(base)         IORD(base, 2)
#define IOWR_ALTERA_AVALON_CF_IDE_SECTOR_COUNT(base, data)   IOWR(base, 2, data)

#define IOADDR_ALTERA_AVALON_CF_IDE_SECTOR_NUMBER(base)      __IO_CALC_ADDRESS_NATIVE(base, 3)
#define IORD_ALTERA_AVALON_CF_IDE_SECTOR_NUMBER(base)        IORD(base, 3)
#define IOWR_ALTERA_AVALON_CF_IDE_SECTOR_NUMBER(base, data)  IOWR(base, 3, data)

#define IOADDR_ALTERA_AVALON_CF_IDE_CYLINDER_LOW(base)       __IO_CALC_ADDRESS_NATIVE(base, 4)
#define IORD_ALTERA_AVALON_CF_IDE_CYLINDER_LOW(base)         IORD(base, 4)
#define IOWR_ALTERA_AVALON_CF_IDE_CYLINDER_LOW(base, data)   IOWR(base, 4, data)

#define IOADDR_ALTERA_AVALON_CF_IDE_CYLINDER_HIGH(base)      __IO_CALC_ADDRESS_NATIVE(base, 5)
#define IORD_ALTERA_AVALON_CF_IDE_CYLINDER_HIGH(base)        IORD(base, 5)
#define IOWR_ALTERA_AVALON_CF_IDE_CYLINDER_HIGH(base, data)  IOWR(base, 5, data)

#define IOADDR_ALTERA_AVALON_CF_IDE_DEVICE_HEAD(base)        __IO_CALC_ADDRESS_NATIVE(base, 6)
#define IORD_ALTERA_AVALON_CF_IDE_DEVICE_HEAD(base)          IORD(base, 6)
#define IOWR_ALTERA_AVALON_CF_IDE_DEVICE_HEAD(base, data)    IOWR(base, 6, data)

#define IOADDR_ALTERA_AVALON_CF_IDE_STATUS(base)             __IO_CALC_ADDRESS_NATIVE(base, 7)
#define IOADDR_ALTERA_AVALON_CF_IDE_COMMAND(base)            __IO_CALC_ADDRESS_NATIVE(base, 7)
#define IORD_ALTERA_AVALON_CF_IDE_STATUS(base)               IORD(base, 7)
#define IOWR_ALTERA_AVALON_CF_IDE_COMMAND(base, data)        IOWR(base, 7, data)

#define IOADDR_ALTERA_AVALON_CF_IDE_ALTERNATE_STATUS(base)   __IO_CALC_ADDRESS_NATIVE(base, 14)
#define IOADDR_ALTERA_AVALON_CF_IDE_DEVICE_CONTROL(base)     __IO_CALC_ADDRESS_NATIVE(base, 14)
#define IORD_ALTERA_AVALON_CF_IDE_ALTERNATE_STATUS(base)     IORD(base, 14)
#define IOWR_ALTERA_AVALON_CF_IDE_DEVICE_CONTROL(base, data) IOWR(base, 14, data)


/* CTL slave I/O RD/WR macros */
#define IORD_ALTERA_AVALON_CF_CTL_STATUS(base)               IORD(base, 0)
#define IOWR_ALTERA_AVALON_CF_CTL_CONTROL(base, data)        IOWR(base, 0, data)

#define IORD_ALTERA_AVALON_CF_IDE_CTL(base)                  IORD(base, 1)
#define IOWR_ALTERA_AVALON_CF_IDE_CTL(base, data)            IOWR(base, 1, data)

/* CTL slave register bit-masks & bit-offsets */
#define ALTERA_AVALON_CF_CTL_STATUS_PRESENT_MSK       (0x1)
#define ALTERA_AVALON_CF_CTL_STATUS_PRESENT_OFST      (0)
#define ALTERA_AVALON_CF_CTL_STATUS_POWER_MSK         (0x2)
#define ALTERA_AVALON_CF_CTL_STATUS_POWER_OFST        (1)
#define ALTERA_AVALON_CF_CTL_STATUS_RESET_MSK         (0x4)
#define ALTERA_AVALON_CF_CTL_STATUS_RESET_OFST        (2)
#define ALTERA_AVALON_CF_CTL_STATUS_IRQ_EN_MSK        (0x8)
#define ALTERA_AVALON_CF_CTL_STATUS_IRQ_EN_OFST       (3)

#define ALTERA_AVALON_CF_IDE_CTL_IRQ_EN_MSK           (0x1)
#define ALTERA_AVALON_CF_IDE_CTL_IRQ_EN_OFST          (0)



#endif // _DOSFS_H
