#include "config.h"
#include "cpuio.h"
#include "cpu.h"
#include "cdefBF537.h"

/* cpuInit():
 * This function is called at CPU reset time to initialize the basic
 * stuff on the BF537.  It is called VERY early in the startup, so
 * don't assume printf is available.
 */
void
cpuinit(void)
{
	volatile int c;
	volatile unsigned short stmp;

	/* PLL setup:
	 */
	asm("ssync;");
	*pPLL_CTL = (MSEL_VAL << 9) | PLL_DF_VAL;
	*pSIC_IWR = 1;
	asm("ssync;");
	asm("cli r0;");
	asm("idle;");
	asm("sti r0;");
	asm("ssync;");

	*pPLL_LOCKCNT =	0x0300;
	*pPLL_DIV = (SSEL_VAL) | (CSEL_VAL << 4);
	asm("ssync;");

	*pDTEST_COMMAND = 0;

	/* Memory bank init:
	 * All banks are tied to the 1 4Mg flash, so they're all configured
	 * the same...
	 *   - BWAT: 8 cycles
	 *   - BRAT: 4 cycles
	 *   - BHT:  1 cycle 
	 *   - BST:  2 cycles 
	 *   - BTT:  1 cycle 
	 */
#if PLATFORM_PCORE537
	*pEBIU_AMBCTL0 = 0x7bb07bb0;
	*pEBIU_AMBCTL1 = 0x7bb07bb0;
#else
	*pEBIU_AMBCTL0 = 0x84668466;
	*pEBIU_AMBCTL1 = 0x84668466;
#endif
	asm("ssync;");

	/* SDRAM ctrlr init:
	 * (should be done 100us after reset)
	 * Refer to pg 6-60 of HW reference.
	 */
	for(c=0;c<100000;c++);
	*pEBIU_SDRRC = (((SYSCLKFREQ/1000) * 64) / 8192) - (TRAS + TRP);
	asm("ssync;");
	*pEBIU_SDBCTL = (EBCAW_10 | EBSZ_64 | EBE);
	asm("ssync;");
	if (*pEBIU_SDSTAT & SDRS)
		*pEBIU_SDGCTL = (PSS | TWR_2 | TRCD_2 | TRP_2 | TRAS_5 | CL_3 | SCTLE);
	else
		*pEBIU_SDGCTL = (TWR_2 | TRCD_2 | TRP_2 | TRAS_5 | CL_3 | SCTLE);
	asm("ssync;");

	*pEBIU_AMGCTL = 0x00ff;
	asm("ssync;");
	
	for(c=0;c<10000;c++);
		*(char *)c = (char)c;

	*pPORT_MUX = 0x0000;

#if PLATFORM_VTCEZ
	/* Port F configuration:
	 * Mix of GPIO and special function...
	 */
	stmp = *pPORTF_FER;
	*pPORTF_FER = 0x838f;
	*pPORTF_FER = 0x838f;
	*pPORTFIO_DIR = 0x7470;

	/* Port G configuration:
	 * All bits configured as PPI (FER bits=1)
	 */
	stmp = *pPORTG_FER;
	*pPORTG_FER = 0xffff;
	*pPORTG_FER = 0xffff;

#else

	/* Port F configuration:
	 */
	stmp = *pPORTF_FER;
	*pPORTF_FER = 0x000f;
	*pPORTF_FER = 0x000f;
	*pPORTFIO_DIR = LED_BITMASK;

	/* Port G configuration:
	 */
	stmp = *pPORTG_FER;
	*pPORTG_FER = 0x0000;
	*pPORTG_FER = 0x0000;

#endif

	/* Port H configuration:
	 */
	stmp = *pPORTH_FER;
	*pPORTH_FER = 0xffff;
	*pPORTH_FER = 0xffff;

	/* Enable the RTC pre-Scaler:
	 */
	*pRTC_PREN = 1;
}

void
icache_off(void)
{
	int i;
	volatile unsigned long *icplb;

	/* Disable I-cache...
	 */
	*pIMEM_CONTROL = 0x00000000;
	asm("ssync;");

	icplb = pICPLB_DATA0;
	for(i=0;i<16;i++) {
		*icplb++ = 0;
		asm("csync;");
	}

	asm("ssync;");
}

void
dcache_off(void)
{
	int i;
	volatile unsigned long *dcplb;

	/* Disable I-cache...
	 */
	*pDMEM_CONTROL = 0x00000000;
	asm("ssync;");

	dcplb = pDCPLB_DATA0;
	for(i=0;i<16;i++) {
		*dcplb++ = 0;
		asm("csync;");
	}

	asm("ssync;");
}

/* bootinit(), ramtstinit() & braminit():
 * One of these functions is called for each of the three different
 * builds created for the BF537.  Refer to reset.S.
 */
void
bootinit(void)
{
}

void
ramtstinit(void)
{
}

void
braminit(void)
{
#if 0
	extern void icache_off(void);

	icache_off();

	/* Enable I-cache for the top 1Mg of SDRAM used by uMon
	 */
	*pICPLB_DATA0 = 0x00021105;
	*pICPLB_ADDR0 = (void *)0x03f00000;
	*pIMEM_CONTROL = 0x00000007;
	asm("ssync;");
#endif
}

