/*
 * Monitor configuration file for CSB226
 *
 * Adapted by Mike Kelly, Cogent Computer Systems, Inc. email: mike@cogcomp.com
 *
 *
 * General notice:
 * This code is part of a boot-monitor package developed as a generic base
 * platform for embedded system designs.  As such, it is likely to be
 * distributed to various projects beyond the control of the original
 * author.  Please notify the author of any enhancements made or bugs found
 * so that all may benefit from the changes.  In addition, notification back
 * to the author will allow the new user to pick up changes that may have
 * been made by other users after this version of the code was distributed.
 *
 * Author:  Ed Sutter
 * email:   esutter@lucent.com
 * phone:   908-582-2351
 */

/* DEFAULT_ETHERADD & DEFAULT_IPADD:
 * Refer to notes in ethernet.c function EthernetStartup() for details
 * regarding the use of these definitions.
 * DEFAULT_IPADD could be set to "DHCP" or "BOOTP" as well.
 */
#define     DEFAULT_ETHERADD "00:30:23:00:00:01"  	// Cogent Block
#define     DEFAULT_IPADD    "192.168.0.214"		

#define CPU_LE
#define MONSTACKSIZE (8*1024)

// override the app ram base to set it to 3Mbyte.  This reserves space
// for umon and the LCD controller buffer
// #define APPRAMBASE_OVERRIDE 0xa0300000

// Force the initialization of BSS
//#define FORCE_BSS_INIT


/* Defining DONT_CENTER_MONHEADER eliminates the automatic centering
 * of the monitor's startup banner...
 */
#define DONT_CENTER_MONHEADER

/* XBUFCNT & RBUFCNT:
 *  Number of transmit and receive buffers allocated to ethernet.
 *  The total of XBUFCNT+RBUFCNT should not exceed MAXEBDS
 */
#define XBUFCNT 	8
#define RBUFCNT 	8
#define XBUFSIZE	2048
#define RBUFSIZE	2048

/* LOOPS_PER_SECOND:
 * Approximately the size of a loop that will cause a 1-second delay.
 * This can be guestimated or modified with the sleep -c command at the
 * monitor command line.
 */
#define LOOPS_PER_SECOND    20000

/* The CSB272 hardware simply has one bank of flash devices.
 * We define SINGLE_FLASH_DEVICE here to pull in the default
 * code from the flash driver file.
 */
#define SINGLE_FLASH_DEVICE		

/* DHCP_RETRYCOUNT is used as a timeout for DHCP or BOOTP startup.
 * It is kept in the cpu.h file because it is target dependent loop time.
 */
#define DHCP_RETRYCOUNT     500000

/* Flash bank configuration:
#define FLASHRAM_BASE			0xa0200000
#define FLASHRAM_END			0xa02fffff
#define FLASHRAM_SECTORSIZE		0x010000
#define FLASHRAM_SECTORCOUNT	16
#define FLASHRAM_BANKNUM		1
 */

#ifdef FLASHRAM_BASE
#define FLASHBANKS              2
#else
#define FLASHBANKS              1
#endif
#define FLASH_BANK0_WIDTH       4
#define FLASH_BANK0_BASE_ADDR   0x0
#define FLASH_LARGEST_SECTOR    0x40000
#define FLASH_COPY_TO_RAM	    
#define FLASH_TIMEOUT			100000

/* TFS definitions:
 *  TFSSTART:       Base address in FLASH at which TFS starts.
 *  TFSEND:         End address of TFS in FLASH.
 *  TFSSPARE:       Location of sector that is used as the spare sector
 *                  by TFS for defragmentation.
 *  TFSSPARESIZE:   Size of the spare sector used by TFS for defragmentation.
 *  TFSSECTORCOUNT: Number of eraseable sectors that TFS covers, not including
 *                  the TFSSPARE sector.
 */
#define TFSSPARESIZE    FLASH_LARGEST_SECTOR
#define TFS_DEVTOT      1
#define TFSSTART        (FLASH_BANK0_BASE_ADDR+0x80000)
#define TFSEND          (FLASH_BANK0_BASE_ADDR+0x00fbffff)
#define TFSSPARE        (TFSEND+1)
#define TFSSECTORCOUNT	((TFSSPARE-TFSSTART)/0x40000)
#define TFS_EBIN_ELFMSBIN	1
#define TFS_VERBOSE_STARTUP	1
#define TFS_AUTOBOOT_ABORTABLE	    

/* FLASH_PROTECT_RANGE is used by the flash code to determine how much of the
 * boot flash is software-protected.  It is usually something to
 * cover the size of the monitor executable.
 */
#define FLASH_PROTECT_RANGE		"0-1"
#define LOCK_FLASH_PROTECT_RANGE

/* Specify CPU/PLATFORM type and name so that common code can be used
 * for a similar cpu, on different platforms.
 * The 'TYPE' definition is used for ifdefs in the code and the 'NAME'
 * is used for printfs in the code.
 */
#define CPU_TYPE        PXA250
#define CPU_NAME        "PXA250 XScale"
#define PLATFORM_TYPE   CSB226
#define PLATFORM_NAME   "Cogent CSB226"

/* Specify the size of the memory block (in monitor space) that is to be
 * allocated to malloc in the monitor.  Note that this size can be dynamically
 * increased using the heap extension option in the heap command.
 */
#define ALLOCSIZE 64*1024

/* Specify inclusion of subsystems within the monitor here.
 * Refer to comments in common/monitor/inc_check.h for details on
 * each of these macros.
 */
#define INCLUDE_PS2KBD			0

#define INCLUDE_MEMTRACE		1
#define INCLUDE_MEMCMDS         1
#define INCLUDE_EDIT            1
#define INCLUDE_DISASSEMBLER    1
#define INCLUDE_UNZIP           1
#define INCLUDE_ETHERNET        1
#define INCLUDE_ICMP			1
#define INCLUDE_TFTP            1
#define INCLUDE_DHCPBOOT        1
#define INCLUDE_TFS             1
#define INCLUDE_TFSCLI          1
#define INCLUDE_TFSAPI          1
#define INCLUDE_TFSSCRIPT       1
#define INCLUDE_TFSSYMTBL       1
#define INCLUDE_XMODEM          1
#define INCLUDE_LINEEDIT        1
#define INCLUDE_EE              1
#define INCLUDE_FLASH           1
#define INCLUDE_MALLOC			1
#define INCLUDE_SHELLVARS		1
#define INCLUDE_STRACE          1
#define INCLUDE_CAST            1
#define INCLUDE_STRUCT          1
#define INCLUDE_REDIRECT        1
#define INCLUDE_QUICKMEMCPY     1
#define INCLUDE_PROFILER        1
#define INCLUDE_BBC             0
#define INCLUDE_STOREMAC		0
#define INCLUDE_PORTCMD			0
#define INCLUDE_HWTMR			0
#define INCLUDE_SYSLOG			1
#define INCLUDE_VERBOSEHELP		0
#define INCLUDE_USRLVL			1

#if INCLUDE_PS2KBD
#define MORE_PUTCHAR	sed_putchar
#define MORE_GOTACHAR	ps2_gotachar
#define MORE_GETCHAR 	ps2_getchar
#endif

#define CONSOLE_UART_BASE	0x40100000	/* See UART1_BASE in cpuio.h */
#define SIO_STEP			4
#define IEN_DEFAULT			0x40
#define MCTL_DEFAULT		0x01

/* Inclusion of this next file will make sure that all of the above
 * inclusions are legal; and warn/adjust where necessary.
 */
#include "inc_check.h"
