/* asm/arch/memconfig.h */

#ifndef __ASM_ARCH_MEMCONFIG_H_
#define __ASM_ARCH_MEMCONFIG_H_

/* Specify Platform */

#define CADENUX_DM320_TI 1
#define CADENUX_DM320_TI_5 1
#define CADENUX_USE_MMU 1

/* FLASH Resources */

#define CADENUX_NFLASH_PARTS 1

#define SRAM_FOR_FLASH 0
#define TOSHIBA_TC58FB160FT 1
#define FUJITSU_MBM29DL323B 2
#define INTEL_28F128 3
#define INTEL_28F160 4
#define INTEL_28F320 5
#define INTEL_28F320x2 6
#define AMD_AM29DL164DT 7
#define AMD_AM29L640Dx2 8
#define AMD_AM29DL323DT 9
#define AMD_AM29DL323DB 10
#define AMD_AM29BDS128x2 11
#define FUJITSU_29LV320T 12
#define HYNIX_HY29LD320B 13
#define CFI_CMDSET_2 14
#define MACRONIX_MX29LV160BT 15
#define CADENUX_FLASH_TYPE TOSHIBA_TC58FB160FT
#undef CADENUX_FLASH2_TYPE

#define CADENUX_FLASH_BASE 0x00100000
#define CADENUX_FLASH_VADDR 0xd0000000
#undef CADENUX_FLASH2_BASE

#define CADENUX_FLASH_SIZE 0x00200000
#undef CADENUX_FLASH2_SIZE
#define FLASH_MEM_BASE CADENUX_FLASH_BASE
#define FLASH_SIZE CADENUX_FLASH_SIZE

/* Compression settings */

#define CADENUX_KERNEL_COMPRESSED 1
#define CADENUX_FS_COMPRESSED 0

/* FLASH Memory Map */

#define CADENUX_FLASH_OFFSET 0x00000000

#define CADENUX_BTLDR_FLASH_START 0x00100000
#define CADENUX_BTLDR_FLASH_OFFSET 0x00000000
#define CADENUX_BTLDR_FLASH_SIZE 0x00020000

#define CADENUX_PARAMS_FLASH_START 0x00120000
#define CADENUX_PARAMS_FLASH_OFFSET 0x00020000
#define CADENUX_PARAMS_FLASH_SIZE 0x00010000

#define CADENUX_KERNEL_FLASH_START 0x00130000
#define CADENUX_KERNEL_FLASH_OFFSET 0x00030000
#define CADENUX_KERNEL_FLASH_SIZE 0x000a0000

#define CADENUX_FS_FLASH_START 0x001d0000
#define CADENUX_FS_FLASH_OFFSET 0x000d0000
#define CADENUX_FS_FLASH_SIZE 0x00130000

/* SDRAM Resources */

#define SDRAM_TYPE_UNKNOWN 0
#define SDRAM_TYPE_D4564163 1
#define SDRAM_TYPE_K4S561632D 2
#define SDRAM_TYPE_TC55V16256 3
#define SDRAM_TYPE_K4S281633D 4
#define SDRAM_TYPE_K4S643233F 5
#define SDRAM_TYPE_K4M563233D 6
#define SDRAM_TYPE_HY57V653220 7
#define SDRAM_TYPE_HY5V52C 8
#define CADENUX_SDRAM_TYPE SDRAM_TYPE_UNKNOWN

#define CADENUX_SDRAM_BASE 0x01000000
#define CADENUX_SDRAM_SIZE 0x02000000

/* SDRAM Memory Map */

#define CADENUX_BTLDR_SDRAM_START 0x01150000
#define CADENUX_BTLDR_SDRAM_END 0x01180000
#define CADENUX_KERNEL_SDRAM_START 0x01008000
#define CADENUX_FS_SDRAM_START 0x02d00000

/* IRAM Memory Map */

#undef CADENUX_USE_IRAM
#define CADENUX_IRAM_BASE 0x00000004

/* Bootloader Settings */

#define CADENUX_BTLDR_RRLOAD 0
#define CADENUX_BTLDR_UMON 1
#define CADENUX_BTLDR_REDBOOT 2
#define CADENUX_BTLDR CADENUX_BTLDR_RRLOAD

#undef CADENUX_BTLDR_FAST_BOOT
#undef CADENUX_BTLDR_NO_MENUS
#undef CADENUX_BTLDR_DOC_SUPPORT
#undef CADENUX_BTLDR_SMARTMEDIA_SUPPORT
#define CADENUX_BTLDR_MEMORY_DEBUG 1
#define CADENUX_BTLDR_MEMMAP_DEBUG 1
#undef CADENUX_BTLDR_CS8900_DEBUG
#undef CADENUX_BTLDR_SMC91111_DEBUG
#undef CADENUX_BTLDR_DM9000_DEBUG
#undef CADENUX_BTLDR_FLASH_DEBUG

/* Linux Kernel */

#undef CADENUX_BOOT_FROM_FLASH
#define CADENUX_BOOT_FROM_SDRAM 1
#define CADENUX_KERNEL_IMAGE_START 0x01008000
#define CADENUX_KERNEL_TEXTADDR 0xc0008000
#define CADENUX_KERNEL_STACK 0x00007800
#define CADENUX_KERNEL_PARMS 0x01150000

/* Linux Root File System */

#define ROOTFS_ROMFS 0
#define ROOTFS_CRAMFS 1
#define ROOTFS_JFFS 2
#define ROOTFS_JFFS2 3
#define ROOTFS_EXT2FS 4
#define ROOTFS_EXT3 5
#define ROOTFS_NFS 6

#define CADENUX_ROOT_FS ROOTFS_EXT2FS

#define RRFLASHFS_OTHER 0
#define RRFLASHFS_JFFS 1
#define RRFLASHFS_JFFS2 2
#define RRFLASHFS_INITRD 3

#define CADENUX_RRFLASH_FS RRFLASHFS_INITRD

#define CADENUX_FLASH_FS 0
#define CADENUX_USE_RAMDISKS 0
#define CADENUX_FILESYS_START 0x02d00020
#define CADENUX_FILESYS_VADDR 0xc1d00020
#define CADENUX_FILESYS_SIZE 0x002fffe0
#define CADENUX_RAMDISK_SIZE 256

/* Linux Memory Pool */

#define CADENUX_DRAM_BASE 0x01000000
#define CADENUX_PHYS_OFFSET 0x01000000
#define CADENUX_DRAM_SIZE 0x01d00000

/* Device Description */

#define CADENUX_ETHERNET_BASE 0x60000300

#define ETHERNET_TYPE_NONE 0
#define ETHERNET_TYPE_UNKNOWN 1
#define ETHERNET_TYPE_CS8900 2
#define ETHERNET_TYPE_SMC9194 3
#define ETHERNET_TYPE_SMC91C111 4
#define ETHERNET_TYPE_DM9000 5
#define CADENUX_ETHERNET_TYPE ETHERNET_TYPE_CS8900

#define ETHERNET_PHY_UNKNOWN 0
#define ETHERNET_PHY_LU3X31T_T64 1
#define ETHERNET_PHY_AC101L 2
#define CADENUX_ETHERNET_PHY ETHERNET_PHY_UNKNOWN
#undef CADENUX_USB_CONSOLE
#define CADENUX_SERIAL_CONSOLE_BAUD 115200
#undef CADENUX_ALT_SERIAL_CONSOLE

/* Networking */

#define CADENUX_NETWORK_TARGET_IP "10.0.0.32"
#define CADENUX_NETWORK_TARGET_NAME "dm320-evm"
#define CADENUX_NETWORK_TARGET_MAC "00:e0:de:ad:be:f0"
#define CADENUX_NETWORK_TARGET_MASK "255.255.255.0"
#define CADENUX_NETWORK_TARGET_BCAST "10.0.0.255"
#define CADENUX_NETWORK_TARGET_GW_IP "10.0.0.1"
#define CADENUX_NETWORK_TARGET_NET_IP "10.0.0.0"
#define CADENUX_NETWORK_SOURCE_IP "10.0.0.3"
#define CADENUX_NETWORK_SOURCE_NAME "sax-lx"
#define CADENUX_NETWORK_SOURCE_MAC "00:E0:7D:F1:18:A1"

#endif /* __ASM_ARCH_MEMCONFIG_H_ */
