//=============================================================================
//
//      cpuio.h
//
//      CPU/Board Specific IO
//
// Author(s):    Michael Kelly, Cogent Computer Systems, Inc.
// Date:         05-26-2002
// Description:  This file contains the IO functions required by Micro Monitor
//				 that are unique to each CPU/Board combination
//
// Modified by Technology Rescue.
//	12.03.2003 : Changed PS2_REG macro (use PS2_STEP).
//	09.05.2003 : Added LCD defs here.
//	03.06.2003 : Added LCD addresses for WindowsCE.
//	06.06.2003 : Changed LCD addresses for WindowsCE.
//	11.07.2003 : Added file names.
//	21.08.2003 : changed file name defs.
//	31.10.2003 : changed 0xA1E57000 to 0xA3E57000 to support 64MBytes SDRAM size.
//	21.11.2003 : changed 0xA1000000 to 0xA3000000.
//	25.11.2003 : added RTC RAM defs.
//
//=============================================================================


#include "pxa250.h"

// board specific defines for micro monitor
#define DEFAULT_BAUD_RATE   	38400
#define MON_CPU_CLOCK       	33000000

#define LOOPS_PER_USEC			67

// ps2 keyboard/mouse controller
#define PS2_BASE				0x04000000		// CS1
#define PS2_STEP				4
#define PS2_REG(_x_)  			*(vuchar *)(PS2_BASE + (_x_ * PS2_STEP))
#define PS2_TIMEOUT				100000

// 16550 style DUART - use the defines in 16550.h
// DUART - Channel 1
#define UART1_BASE			0x40100000
#define UARTA(_x_)  			*(vuchar *)(UART1_BASE + _x_)
#define SIO_STEP				4			// internal pxa250 UART is byte wide on 32-bit boundries

// CS8900a 10Mbit Ethernet 
#define CS8900A_BASE_ADDRESS	0x08000300	// CS2
#define CS8900A_STEP			4			// CS8900a is on 32-bit boundries
#define CS_TIMEOUT 				500000		// Time to wait for chip to complete various commands

// LCD defines.
#define LCD_BKL					17			// GPIO 17 controls the LCD Backlight, 1 = On

// Define following to include debug traces.
//#define LCD_DBG

//--------------------------------------------------------------------------
// The LCD frame buffer for Windows CE is fixed at 0xA3E57000.
//
#define LCD_FDR_ADD_CE			0xA3E57000
#define LCD_FDR_CE(_x_)			*(vulong *)(LCD_FDR_ADD_CE + _x_)	// Frame Descriptor, Frame Buffer

#define LCD_PAL_ADD_CE			0xA3E57080
#define LCD_PAL_CE(_x_)			*(vushort *)(LCD_PAL_ADD_CE + _x_)	// Palette Buffer

#define LCD_PDR_ADD_CE			0xA3E57060
#define LCD_PDR_CE(_x_)			*(vulong *)(LCD_PDR_ADD_CE + _x_)	// Palette Descriptor, Palette Buffer

#define LCD_BUF_ADD_CE			0xA3E57280
#define LCD_BUF_CE(_x_)			*(vushort *)(LCD_BUF_ADD_CE + _x_)	// Frame Buffer

//--------------------------------------------------------------------------
// The LCD frame buffer for Umon is fixed at 0xA3000000.
//
#define LCD_FDR_ADD				0xA3000000
#define LCD_FDR(_x_)			*(vulong *)(LCD_FDR_ADD + _x_)	// Frame Descriptor, Frame Buffer

#define LCD_PAL_ADD				0xA3000080
#define LCD_PAL(_x_)			*(vushort *)(LCD_PAL_ADD + _x_)	// Palette Buffer

#define LCD_PDR_ADD				0xA3000060
#define LCD_PDR(_x_)			*(vulong *)(LCD_PDR_ADD + _x_)	// Palette Descriptor, Palette Buffer

#define LCD_BUF_ADD				0xA3000280
#define LCD_BUF(_x_)			*(vushort *)(LCD_BUF_ADD + _x_)	// Frame Buffer
#include "lq64d343.h"

#define CE_FILE_NAME			"nk.nb0"
#define CE_ZIP_NAME				"nk.nbz"
#if INCLUDE_UNZIP
#define FILE_NAME				CE_ZIP_NAME
#else
#define FILE_NAME				CE_FILE_NAME
#endif

// Following defs are used to store IP address.
#define	IP_SIZE					4
#define	IP_ADDR_NAME			"IPADD"
#define	IP_ADDR_IDX				0
#define IP_ADDR_OFFSET			DS1307_RAM_START
#define IP_ADDR_SIZE			IP_SIZE
#define BOOTP_ID				0x50544F42
#define DHCP_ID					0x50434844

// Following defs are used to store netmask.
#define	NETMASK_NAME			"NETMASK"
#define NETMASK_IDX				1
#define NETMASK_OFFSET			(IP_ADDR_OFFSET + IP_ADDR_SIZE)
#define NETMASK_SIZE			IP_SIZE

// Following defs are used to store Gateway IP address.
#define GIPADD_NAME				"GIPADD"
#define GIPADD_IDX				2
#define GIPADD_OFFSET			(NETMASK_OFFSET + NETMASK_SIZE)
#define GIPADD_SIZE				IP_SIZE

// Following defs are used to store primary DNS address.
#define DNS1_NAME				"DNS1"
#define DNS1_IDX				3
#define DNS1_OFFSET				(GIPADD_OFFSET + GIPADD_SIZE)
#define DNS1_SIZE				IP_SIZE

// Following defs are used to store primary DNS address.
#define DNS2_NAME				"DNS2"
#define DNS2_IDX				4
#define DNS2_OFFSET				(DNS1_OFFSET + DNS1_SIZE)
#define DNS2_SIZE				IP_SIZE

// Following defs are used to store Host IP address for WinCE remote display.
#define HOSTIP_NAME				"HOSTIP"
#define HOSTIP_IDX				5
#define HOSTIP_OFFSET			(DNS2_OFFSET + DNS2_SIZE)
#define HOSTIP_SIZE				IP_SIZE

// Following defs are used to store Host IP address for WinCE remote display.
#define ETHERADD_NAME			"ETHERADD"
#define ETHERADD_IDX			6
#define ETHERADD_OFFSET			(HOSTIP_OFFSET + HOSTIP_SIZE)
#define ETHERADD_SIZE			6

#define MAX_IDX					ETHERADD_IDX + 1
#define CONFIG_RAM_SIZE			IP_ADDR_SIZE + NETMASK_SIZE + GIPADD_SIZE + DNS1_SIZE + DNS2_SIZE + HOSTIP_SIZE + ETHERADD_SIZE

// Contains a pointer to the actual boot args structure
#define TCPIP_PTR_OFFSET	   	0x00000050

