/* lq64d343.h

    Author: David de la Cruz, Technology Rescue.
    email:  ddlc@techresc.net

	This file provides SHARP LQ64D343 LCD definitions to use with pxa250_lcd driver.
	This file can be included in board include file.

	Modification History :

	Version 1.0:
		06.05.2003 : new.
		01.10.2003 : fixed missed lines at top screen = LCCR2_BFW should be 2.
 */

#include "pxa270_lcd.h"

#define PIXELS_PER_ROW			640
#define PIXELS_PER_COL			480

#define FONT_WIDTH				8
#define FONT_HEIGHT 			16


#define ROWS_PER_SCREEN			30
#define COLS_PER_SCREEN			80

#define BIT_PER_PIXEL			16			// 16 BPP.
#define LCD_BG_DEF				0x001F		// Light Blue.
#define LCD_FG_DEF				0xFFFF		// White.

#define POWERON_DELAY			40000

/* Setup Timing Registers
 * Note, LCD pixel clock is derived from MEMCLK which
 * defaults to 99.5Mhz.  Any other value requires changing
 * LCCR3_PCD.
 *
 * To support this LCD, MEMCLK should be 147.5MHz or 132.7 MHz.
 */

// 16-bits/pixel.
// Pixel Clock Polarity, 1 = LCD Data is Sampled on Falling Edge.
// Horizontal Sync Polarity, 1 = L_LCLK is Active Low.
// Vertical Sync Polarity, 1= L_FCLK is Active Low.
// Pixel Clock Divisor, PCLK = LCLK/(2*(PCD+1)) = 99.5Mhz/4 = 25Mhz
#define LCCR3_VAL 	  (LCCR3_BPP_16 | LCCR3_PCP | LCCR3_HSP | LCCR3_VSP | LCCR3_PCD(0x01))


// Beginning-of-frame line clock wait count = 3 line.
// End-of-frame line clock wait count = 32 lines.
// Vertical sync pulse width = 32 lines.
// Lines per panel = LPP + 1
#define LCCR2_VAL 	  (LCCR2_BFW(2) | LCCR2_EFW(31) | LCCR2_VSW(31) | LCCR2_LPP((PIXELS_PER_COL - 1)))

// Beginning-of-Line pixel clock Wait count = 1 line.
// End-of-line pixel clock wait count = 161 lines
// Horizontal sync pulse width = 1 : LCD in DE mode.
// Pixels per line = PPL + 1
#define LCCR1_VAL 	  (LCCR1_BLW(0) | LCCR1_ELW(160) | LCCR1_HSW(0) | LCCR1_PPL((PIXELS_PER_ROW - 1)))


// Output FIFO Underrun Mask, 1 = no interrupt.
// Branch Mask, 1 = no interrupt
// Palette DMA Request delay = 0 : disable palette function.
// Quick Disable Interrupt Mask, 1 = no interrupt.
// Passive/Active Display select 1 = TFT.
// End of Frame Interrupt Mask, 1 = no interrupt
// Input FIFO Underrun Mask, 1 = no interrupt
// Start of Frame Mask, 1 = no interrupt
// LCD Disable Done Mask, 1 = no interrupt
#define LCCR0_VAL 	  (LCCR0_OUM | LCCR0_BM | LCCR0_PDD(0) | LCCR0_QDM | LCCR0_PAS | LCCR0_EFM | LCCR0_IUM | LCCR0_SFM | LCCR0_LDM )



