//mx31_iomux.c

#include "config.h"
#include "cpuio.h"
#include "stddefs.h"
#include "genlib.h"
#include "mx35.h"
#include "mx35_iomux.h"
#include "cpu_gpio.h"    // pull in target board specific header
#include "ad7843.h"

//#define SION 0x10
#define SION 0x00


void iomux_init()
{

	// Set up UART3 port
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_ATA_DATA10) = 0x00000001;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_ATA_DATA11) = 0x00000001;

	// Set up contrast signal to GPIO for LCD_BKL
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_CONTRAST) = 0x00000005;

	// Set up for GPIO0 to be USR_LED
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_ATA_DATA12) = 0x00000005;

	// Set up signals for SD Slot 0.  In MX35 it is esdhc3 port
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_ATA_DIOR) = 0x00000001;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_ATA_DIOW) = 0x00000001;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_ATA_DMACK) = 0x00000001;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_ATA_RESET_B) = 0x00000001;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_ATA_DATA3) = 0x00000001;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_ATA_DATA4) = 0x00000001;

#ifdef AD7843_GPIOMODE
	// Set up SPI1 port in GPIO mode for interface to AD7843
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_CSPI1_MOSI) = 0x00000005;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_CSPI1_MISO) = 0x00000005;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_CSPI1_SS1) = 0x00000005;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_CSPI1_SCLK) = 0x00000005;
#else
	// Set up SPI1 port in GPIO mode for interface to AD7843
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_CSPI1_MOSI) = 0x00000000;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_CSPI1_MISO) = 0x00000000;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_CSPI1_SS1) = 0x00000000;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_CSPI1_SCLK) = 0x00000000;
#endif

	// Set up GPIO_3_5 as *PIRQ
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_MLB_SIG) = 0x00000005;

	// Select correct input in Daisy chain for Rx on UART3
	IOMUX_CTL_REG(UART3_RXD_MUX_SELECT_INPUT) = 0x00000002;
	IOMUX_CTL_REG(GPIO3_IPP_IND_G_IN_5_SELECT_INPUT) = 0x00000001;

	// Set FEC pins to correct mode:
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_FEC_TX_CLK) = 0 | SION;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_FEC_RX_CLK) = 0 | SION;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_FEC_RX_DV)  = 0 | SION;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_FEC_COL)    = 0 | SION;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_FEC_RDATA0) = 0 | SION;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_FEC_RDATA1) = 0 | SION;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_FEC_RDATA2) = 0 | SION;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_FEC_RDATA3) = 0 | SION;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_FEC_TDATA0) = 0;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_FEC_TDATA1) = 0;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_FEC_TDATA2) = 0;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_FEC_TDATA3) = 0;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_FEC_TX_EN)  = 0;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_FEC_MDC)    = 0;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_FEC_MDIO)   = 0;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_FEC_TX_ERR) = 0;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_FEC_RX_ERR) = 0 | SION;
	IOMUX_CTL_REG(SW_MUX_CTL_PAD_FEC_CRS)    = 0 | SION;

#if 1
	IOMUX_CTL_REG(SW_PAD_CTL_PAD_FEC_TX_CLK) = 0x000001c0;
	IOMUX_CTL_REG(SW_PAD_CTL_PAD_FEC_RX_CLK) = 0x000001c0;
	IOMUX_CTL_REG(SW_PAD_CTL_PAD_FEC_RX_DV)  = 0x000001c0;
	IOMUX_CTL_REG(SW_PAD_CTL_PAD_FEC_COL)    = 0x000001c0;
	IOMUX_CTL_REG(SW_PAD_CTL_PAD_FEC_RDATA0) = 0x000001c0;
	IOMUX_CTL_REG(SW_PAD_CTL_PAD_FEC_RDATA1) = 0x000001c0;
	IOMUX_CTL_REG(SW_PAD_CTL_PAD_FEC_RDATA2) = 0x000001c0;
	IOMUX_CTL_REG(SW_PAD_CTL_PAD_FEC_RDATA3) = 0x000001c0;
	IOMUX_CTL_REG(SW_PAD_CTL_PAD_FEC_TDATA0) = 0x00000040;
	IOMUX_CTL_REG(SW_PAD_CTL_PAD_FEC_TDATA1) = 0x00000040;
	IOMUX_CTL_REG(SW_PAD_CTL_PAD_FEC_TDATA2) = 0x00000040;
	IOMUX_CTL_REG(SW_PAD_CTL_PAD_FEC_TDATA3) = 0x00000040;
	IOMUX_CTL_REG(SW_PAD_CTL_PAD_FEC_TX_EN)  = 0x00000040;
	IOMUX_CTL_REG(SW_PAD_CTL_PAD_FEC_MDC)    = 0x00000040;
	IOMUX_CTL_REG(SW_PAD_CTL_PAD_FEC_MDIO)   = 0x000001f0;
	IOMUX_CTL_REG(SW_PAD_CTL_PAD_FEC_TX_ERR) = 0x00000040;
	IOMUX_CTL_REG(SW_PAD_CTL_PAD_FEC_RX_ERR) = 0x000001c0;
	IOMUX_CTL_REG(SW_PAD_CTL_PAD_FEC_CRS)    = 0x000001c0;
#endif
}
