/*********************************************************************
 *
 * Copyright:
 *	MOTOROLA, INC. All Rights Reserved.  
 *  You are hereby granted a copyright license to use, modify, and
 *  distribute the SOFTWARE so long as this entire notice is
 *  retained without alteration in any modified and/or redistributed
 *  versions, and that such modified versions are clearly identified
 *  as such. No licenses are granted by implication, estoppel or
 *  otherwise under any patents or trademarks of Motorola, Inc. This 
 *  software is provided on an "AS IS" basis and without warranty.
 *
 *  To the maximum extent permitted by applicable law, MOTOROLA 
 *  DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED, INCLUDING 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR
 *  PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT WITH REGARD TO THE 
 *  SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF) AND ANY 
 *  ACCOMPANYING WRITTEN MATERIALS.
 * 
 *  To the maximum extent permitted by applicable law, IN NO EVENT
 *  SHALL MOTOROLA BE LIABLE FOR ANY DAMAGES WHATSOEVER (INCLUDING 
 *  WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS 
 *  INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY
 *  LOSS) ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.   
 * 
 *  Motorola assumes no responsibility for the maintenance and support
 *  of this software
 ********************************************************************/

/*
 * File:		nbuf.h
 * Purpose:		Definitions for Network Buffer Allocation.
 *
 * Notes:		These routines implement a static buffer scheme.
 *				The buffer descriptors are as specified by the 
 *				MCF5329 FEC.
 *
 */

#ifndef _NBUF_H
#define _NBUF_H

#include "stddefs.h"

/********************************************************************/

#define Rx	1
#define Tx	0

/*
 * Buffer sizes in bytes: maximum needed for any incoming ethernet
 * packet. 
 */
#define RX_BUFFER_SIZE 1600	/* must be divisible by 16 */
#define TX_BUFFER_SIZE 1600

/* Number of Receive and Transmit Buffers and Buffer Descriptors */
#define NUM_RXBDS 8
#define NUM_TXBDS 8

/* Buffer Descriptor Format */
typedef struct
{
	uint16_t length;	/* transfer length */
	uint16_t status;	/* control and status */
	uint8_t  *data;	/* buffer address */
} NBUF;

/*******************************************************************/

/*
 * Functions to manipulate the network buffers.
 */
void
nbuf_init (void);

uint32_t
nbuf_get_start(uint8_t);

NBUF *
nbuf_rx_allocate (void);

NBUF *
nbuf_tx_allocate (void);

void
nbuf_rx_release (NBUF *);

void
nbuf_tx_release (NBUF *);

int
nbuf_rx_next_ready(void);

int
nbuf_rx_lastinframe(void);

int
nbuf_rx_overflow(void);

void
nbuf_tx_swap(NBUF **, NBUF **);

/********************************************************************/

#endif 	/* _NBUF_H */

