#ifndef __CPU_IPU_H
#define __CPU_IPU_H

#include "mx31.h"

enum {
    IPU_CONF_CSI_EN             = 0x00000001,
    IPU_CONF_IC_EN              = 0x00000002,
    IPU_CONF_ROT_EN             = 0x00000004,
    IPU_CONF_PF_EN              = 0x00000008,
    IPU_CONF_SDC_EN             = 0x00000010,
    IPU_CONF_ADC_EN             = 0x00000020,
    IPU_CONF_DI_EN              = 0x00000040,
    IPU_CONF_DU_EN              = 0x00000080,
    IPU_CONF_PXL_ENDIAN         = 0x00000100,

    FS_PRPVF_ROT_SRC_SEL        = 0x00000040,    
    FS_PRPENC_ROT_SRC_SEL       = 0x00000020,    
    FS_PRPENC_DEST_SEL          = 0x00000010,
    FS_PP_SRC_SEL_MASK          = 0x00000300,
    FS_PP_SRC_SEL_OFFSET        = 8,
    FS_PP_ROT_SRC_SEL_MASK      = 0x00000C00,
    FS_PP_ROT_SRC_SEL_OFFSET    = 10,
    FS_PF_DEST_SEL_MASK         = 0x00003000,
    FS_PF_DEST_SEL_OFFSET       = 12,
    FS_PRPVF_DEST_SEL_MASK      = 0x00070000,
    FS_PRPVF_DEST_SEL_OFFSET    = 16,
    FS_PRPVF_ROT_DEST_SEL_MASK  = 0x00700000,
    FS_PRPVF_ROT_DEST_SEL_OFFSET= 20,
    FS_PP_DEST_SEL_MASK         = 0x07000000,
    FS_PP_DEST_SEL_OFFSET       = 24,
    FS_PP_ROT_DEST_SEL_MASK     = 0x70000000,
    FS_PP_ROT_DEST_SEL_OFFSET   = 28,
    FS_VF_IN_VALID              = 0x00000002,
    FS_ENC_IN_VALID             = 0x00000001,

    FS_SDC_BG_SRC_SEL_MASK      = 0x00000007,
    FS_SDC_BG_SRC_SEL_OFFSET    = 0,
    FS_SDC_FG_SRC_SEL_MASK      = 0x00000070,
    FS_SDC_FG_SRC_SEL_OFFSET    = 4,
    FS_ADC1_SRC_SEL_MASK        = 0x00000700,
    FS_ADC1_SRC_SEL_OFFSET      = 8,
    FS_ADC2_SRC_SEL_MASK        = 0x00007000,
    FS_ADC2_SRC_SEL_OFFSET      = 12,
    FS_AUTO_REF_PER_MASK        = 0x03FF0000,
    FS_AUTO_REF_PER_OFFSET      = 16,

    FS_DEST_ARM                 = 0,
    FS_DEST_ROT                 = 1,
    FS_DEST_PP                  = 1,
    FS_DEST_ADC1                = 2,
    FS_DEST_ADC2                = 3,
    FS_DEST_SDC_BG              = 4,
    FS_DEST_SDC_FG              = 5,
    FS_DEST_ADC                 = 6,

    FS_SRC_ARM                  = 0,
    FS_PP_SRC_PF                = 1,
    FS_PP_SRC_ROT               = 2,

    FS_ROT_SRC_PP               = 1,
    FS_ROT_SRC_PF               = 2,

    FS_PF_DEST_PP               = 1,
    FS_PF_DEST_ROT              = 2,

    FS_SRC_ROT_VF               = 1,
    FS_SRC_ROT_PP               = 2,
    FS_SRC_VF                   = 3,
    FS_SRC_PP                   = 4,
    FS_SRC_SNOOP                = 5,
    FS_SRC_AUTOREF              = 6,
    FS_SRC_AUTOREF_SNOOP        = 7,

    TSTAT_PF_H264_PAUSE         = 0x00000001,
    TSTAT_CSI2MEM_MASK          = 0x0000000C,
    TSTAT_CSI2MEM_OFFSET        = 2,
    TSTAT_VF_MASK               = 0x00000600,
    TSTAT_VF_OFFSET             = 9,
    TSTAT_VF_ROT_MASK           = 0x000C0000,
    TSTAT_VF_ROT_OFFSET         = 18,
    TSTAT_ENC_MASK              = 0x00000180,
    TSTAT_ENC_OFFSET            = 7,
    TSTAT_ENC_ROT_MASK          = 0x00030000,
    TSTAT_ENC_ROT_OFFSET        = 16,
    TSTAT_PP_MASK               = 0x00001800,
    TSTAT_PP_OFFSET             = 11,
    TSTAT_PP_ROT_MASK           = 0x00300000,
    TSTAT_PP_ROT_OFFSET         = 20,
    TSTAT_PF_MASK               = 0x00C00000,
    TSTAT_PF_OFFSET             = 22,
    TSTAT_ADCSYS1_MASK          = 0x03000000,
    TSTAT_ADCSYS1_OFFSET        = 24,
    TSTAT_ADCSYS2_MASK          = 0x0C000000,
    TSTAT_ADCSYS2_OFFSET        = 26,

    TASK_STAT_IDLE              = 0,
    TASK_STAT_ACTIVE            = 1,
    TASK_STAT_WAIT4READY        = 2,
    
    // Register bits
    SDC_COM_TFT_COLOR =     0x00000001UL,
    SDC_COM_FG_EN =         0x00000010UL,
    SDC_COM_GWSEL =         0x00000020UL,
    SDC_COM_GLB_A =         0x00000040UL,
    SDC_COM_KEY_COLOR_G =   0x00000080UL,
    SDC_COM_BG_EN =         0x00000200UL,
    SDC_COM_SHARP =         0x00001000UL,

    SDC_V_SYNC_WIDTH_L =    0x00000001UL,

    ADC_CONF_PRP_EN             = 0x00000001L,
    ADC_CONF_PP_EN              = 0x00000002L,
    ADC_CONF_MCU_EN             = 0x00000004L,

    ADC_DISP_CONF_SL_MASK       = 0x00000FFFL,
    ADC_DISP_CONF_TYPE_MASK     = 0x00003000L,
    ADC_DISP_CONF_TYPE_XY       = 0x00002000L,
    
    ADC_DISP_VSYNC_D0_MODE_MASK = 0x00000003L,
    ADC_DISP_VSYNC_D0_WIDTH_MASK = 0x003F0000L,
    ADC_DISP_VSYNC_D12_MODE_MASK = 0x0000000CL,
    ADC_DISP_VSYNC_D12_WIDTH_MASK = 0x3F000000L,

    // Image Converter Register bits
    IC_CONF_PRPENC_EN       = 0x00000001,
    IC_CONF_PRPENC_CSC1     = 0x00000002,
    IC_CONF_PRPENC_ROT_EN   = 0x00000004,
    IC_CONF_PRPVF_EN        = 0x00000100,
    IC_CONF_PRPVF_CSC1      = 0x00000200,
    IC_CONF_PRPVF_CSC2      = 0x00000400,
    IC_CONF_PRPVF_CMB       = 0x00000800,
    IC_CONF_PRPVF_ROT_EN    = 0x00001000,
    IC_CONF_PP_EN           = 0x00010000,
    IC_CONF_PP_CSC1         = 0x00020000,
    IC_CONF_PP_CSC2         = 0x00040000,
    IC_CONF_PP_CMB          = 0x00080000,
    IC_CONF_PP_ROT_EN       = 0x00100000,
    IC_CONF_IC_GLB_LOC_A    = 0x10000000,
    IC_CONF_KEY_COLOR_EN    = 0x20000000,
    IC_CONF_RWS_EN          = 0x40000000,
    IC_CONF_CSI_MEM_WR_EN   = 0x80000000,

    IDMA_CHAN_INVALID = 0x000000FF,
    IDMA_IC_0 = 0x00000001,
    IDMA_IC_1 = 0x00000002,
    IDMA_IC_2 = 0x00000004,
    IDMA_IC_3 = 0x00000008,
    IDMA_IC_4 = 0x00000010,
    IDMA_IC_5 = 0x00000020,
    IDMA_IC_6 = 0x00000040,
    IDMA_IC_7 = 0x00000080,
    IDMA_IC_8 = 0x00000100,
    IDMA_IC_9 = 0x00000200,
    IDMA_IC_10 = 0x00000400,
    IDMA_IC_11 = 0x00000800,
    IDMA_IC_12 = 0x00001000,
    IDMA_IC_13 = 0x00002000,
    IDMA_SDC_BG = 0x00004000,        
    IDMA_SDC_FG = 0x00008000,        
    IDMA_SDC_MASK = 0x00010000,
    IDMA_SDC_PARTIAL = 0x00020000,
    IDMA_ADC_SYS1_WR = 0x00040000,
    IDMA_ADC_SYS2_WR = 0x00080000,
    IDMA_ADC_SYS1_CMD = 0x00100000,
    IDMA_ADC_SYS2_CMD = 0x00200000,
    IDMA_ADC_SYS1_RD = 0x00400000,
    IDMA_ADC_SYS2_RD = 0x00800000,
    IDMA_PF_QP = 0x01000000,
    IDMA_PF_BSP = 0x02000000,
    IDMA_PF_Y_IN = 0x04000000,
    IDMA_PF_U_IN = 0x08000000,
    IDMA_PF_V_IN = 0x10000000,
    IDMA_PF_Y_OUT = 0x20000000,
    IDMA_PF_U_OUT = 0x40000000,
    IDMA_PF_V_OUT = 0x80000000,

    CSI_SENS_CONF_DATA_FMT_SHIFT        = 8,
    CSI_SENS_CONF_DATA_FMT_RGB_YUV444   = 0x00000000L,
    CSI_SENS_CONF_DATA_FMT_YUV422       = 0x00000200L,
    CSI_SENS_CONF_DATA_FMT_BAYER        = 0x00000300L,

    CSI_SENS_CONF_VSYNC_POL_SHIFT       = 0,
    CSI_SENS_CONF_HSYNC_POL_SHIFT       = 1,
    CSI_SENS_CONF_DATA_POL_SHIFT        = 2,
    CSI_SENS_CONF_PIX_CLK_POL_SHIFT     = 3,
    CSI_SENS_CONF_SENS_PRTCL_SHIFT      = 4,
    CSI_SENS_CONF_SENS_CLKSRC_SHIFT     = 7,
    CSI_SENS_CONF_DATA_WIDTH_SHIFT      = 10,
    CSI_SENS_CONF_EXT_VSYNC_SHIFT       = 15,
    CSI_SENS_CONF_DIVRATIO_SHIFT        = 16,

    PF_CONF_TYPE_MASK                 = 0x00000007,
    PF_CONF_TYPE_SHIFT                = 0,
    PF_CONF_PAUSE_EN                  = 0x00000010,
    PF_CONF_RESET                     = 0x00008000,
    PF_CONF_PAUSE_ROW_MASK            = 0x00FF0000,
    PF_CONF_PAUSE_ROW_SHIFT           = 16,

    /* DI_DISP_SIG_POL bits */
    DI_D3_VSYNC_POL_SHIFT               = 28,
    DI_D3_HSYNC_POL_SHIFT               = 27,
    DI_D3_DRDY_SHARP_POL_SHIFT          = 26,
    DI_D3_CLK_POL_SHIFT                 = 25,
    DI_D3_DATA_POL_SHIFT                = 24,

    /* DI_DISP_IF_CONF bits */
    DI_D3_CLK_IDLE_SHIFT                = 26,
    DI_D3_CLK_SEL_SHIFT                 = 25,
    DI_D3_DATAMSK_SHIFT                 = 24,

    DISPx_IF_CLK_DOWN_OFFSET            = 22,
    DISPx_IF_CLK_UP_OFFSET              = 12,
    DISPx_IF_CLK_PER_OFFSET             = 0,
    DISPx_IF_CLK_READ_EN_OFFSET         = 16,
    DISPx_PIX_CLK_PER_OFFSET            = 0,

    DI_CONF_DISP0_EN                    = 0x00000001L,
    DI_CONF_DISP0_IF_MODE_OFFSET        = 1,
    DI_CONF_DISP0_BURST_MODE_OFFSET     = 3,
    DI_CONF_DISP1_EN                    = 0x00000100L,
    DI_CONF_DISP1_IF_MODE_OFFSET        = 9,
    DI_CONF_DISP1_BURST_MODE_OFFSET     = 12,
    DI_CONF_DISP2_EN                    = 0x00010000L,
    DI_CONF_DISP2_IF_MODE_OFFSET        = 17,
    DI_CONF_DISP2_BURST_MODE_OFFSET     = 20,

    DI_SER_DISPx_CONF_SER_BIT_NUM_OFFSET = 16,
    DI_SER_DISPx_CONF_PREAMBLE_OFFSET   = 8,
    DI_SER_DISPx_CONF_PREAMBLE_LEN_OFFSET = 4,
    DI_SER_DISPx_CONF_RW_CFG_OFFSET     = 1,
    DI_SER_DISPx_CONF_BURST_MODE_EN     = 0x01000000L,
    DI_SER_DISPx_CONF_PREAMBLE_EN       = 0x00000001L,

    /* DI_DISP_ACC_CC */
    DI_DISP_ACC_CC_D0_IF_CLK_CNT_D      = 0x00000003L,
};

/*!
 * Enumeration of IPU rotation modes
 */
typedef enum {
        // Note the enum values correspond to BAM value
        IPU_ROTATE_NONE           = 0,
        IPU_ROTATE_VERT_FLIP      = 1,
        IPU_ROTATE_HORIZ_FLIP     = 2,
        IPU_ROTATE_180            = 3,
        IPU_ROTATE_90_RIGHT       = 4,
        IPU_ROTATE_90_RIGHT_VFLIP = 5,
        IPU_ROTATE_90_RIGHT_HFLIP = 6,
        IPU_ROTATE_90_LEFT        = 7,
} ipu_rotate_mode_t;

/*!
 * Enumeration of Post Filter modes
 */
typedef enum {
    PF_DISABLE_ALL          = 0,
    PF_MPEG4_DEBLOCK        = 1,
    PF_MPEG4_DERING         = 2,
    PF_MPEG4_DEBLOCK_DERING = 3,
    PF_H264_DEBLOCK         = 4,
} pf_operation_t;

/*!
 * Enumeration of Synchronous (Memory-less) panel types
 */
typedef enum {
    IPU_PANEL_SHARP_TFT,
    IPU_PANEL_TFT,
} ipu_panel_t;


/*  IPU Pixel format definitions */
/*  Four-character-code (FOURCC) */
#define ipu_fourcc(a,b,c,d) (((ulong)(a)<<0)|((ulong)(b)<<8)|((ulong)(c)<<16)|((ulong)(d)<<24))

/*!
 * @name IPU Pixel Formats
 *
 * Pixel formats are defined with ASCII FOURCC code. The pixel format codes are
 * the same used by V4L2 API.
 */

/*! @{ */
/*! @name Generic or Raw Data Formats */ /*! @{ */
#define IPU_PIX_FMT_GENERIC ipu_fourcc('I','P','U','0') /*!< IPU Generic Data */
#define IPU_PIX_FMT_GENERIC_32 ipu_fourcc('I','P','U','1') /*!< IPU Generic Data */
/*! @} */

/*! @name RGB Formats */ /*! @{ */
#define IPU_PIX_FMT_RGB332  ipu_fourcc('R','G','B','1') /*!<  8  RGB-3-3-2     */
#define IPU_PIX_FMT_RGB555  ipu_fourcc('R','G','B','O') /*!< 16  RGB-5-5-5     */
#define IPU_PIX_FMT_RGB565  ipu_fourcc('R','G','B','P') /*!< 16  RGB-5-6-5     */
#define IPU_PIX_FMT_RGB666  ipu_fourcc('R','G','B','6') /*!< 18  RGB-6-6-6     */
#define IPU_PIX_FMT_BGR24   ipu_fourcc('B','G','R','3') /*!< 24  BGR-8-8-8     */
#define IPU_PIX_FMT_RGB24   ipu_fourcc('R','G','B','3') /*!< 24  RGB-8-8-8     */
#define IPU_PIX_FMT_BGR32   ipu_fourcc('B','G','R','4') /*!< 32  BGR-8-8-8-8   */
#define IPU_PIX_FMT_BGRA32  ipu_fourcc('B','G','R','A') /*!< 32  BGR-8-8-8-8   */
#define IPU_PIX_FMT_RGB32   ipu_fourcc('R','G','B','4') /*!< 32  RGB-8-8-8-8   */
#define IPU_PIX_FMT_RGBA32  ipu_fourcc('R','G','B','A') /*!< 32  RGB-8-8-8-8   */
#define IPU_PIX_FMT_ABGR32  ipu_fourcc('A','B','G','R') /*!< 32  ABGR-8-8-8-8  */
/*! @} */

/*! @name YUV Interleaved Formats */ /*! @{ */
#define IPU_PIX_FMT_YUYV    ipu_fourcc('Y','U','Y','V') /*!< 16  YUV 4:2:2     */
#define IPU_PIX_FMT_UYVY    ipu_fourcc('U','Y','V','Y') /*!< 16  YUV 4:2:2     */
#define IPU_PIX_FMT_Y41P    ipu_fourcc('Y','4','1','P') /*!< 12  YUV 4:1:1     */
/*! @} */

/*! @name YUV Planar Formats */ /*! @{ */
#define IPU_PIX_FMT_GREY    ipu_fourcc('G','R','E','Y') /*!< 8  Greyscale     */
#define IPU_PIX_FMT_YVU410P ipu_fourcc('Y','V','U','9') /*!< 9  YVU 4:1:0     */
#define IPU_PIX_FMT_YUV410P ipu_fourcc('Y','U','V','9') /*!< 9  YUV 4:1:0     */
#define IPU_PIX_FMT_YVU420P ipu_fourcc('Y','V','1','2') /*!< 12  YVU 4:2:0     */
#define IPU_PIX_FMT_YUV420P ipu_fourcc('I','4','2','0') /*!< 12  YUV420 planar */
#define IPU_PIX_FMT_YUV420P2 ipu_fourcc('Y','U','1','2') /*!< 12  YUV 4:2:0 planar */
#define IPU_PIX_FMT_YVU422P ipu_fourcc('Y','V','1','6') /*!< 16  YVU422 planar */
/*! @} */

//#define IPU_PIX_FMT_YUV422P ipu_fourcc('4','2','2','P') /* 16  YVU422 planar */
//#define IPU_PIX_FMT_YUV411P ipu_fourcc('4','1','1','P') /* 16  YVU411 planar */
//#define IPU_PIX_FMT_YYUV    ipu_fourcc('Y','Y','U','V') /* 16  YUV 4:2:2     */

// IPU Driver channels definitions. 
// Note these are different from IDMA channels
#define _MAKE_CHAN(num, in, out, sec)    ((num << 24) | (sec << 16) | (out << 8) | in) 
#define IPU_CHAN_ID(ch)         (ch >> 24)
#define IPU_CHAN_SEC_DMA(ch)    ((uint32_t) (ch >> 16) & 0xFF)
#define IPU_CHAN_OUT_DMA(ch)    ((uint32_t) (ch >> 8) & 0xFF)
#define IPU_CHAN_IN_DMA(ch)     ((uint32_t) (ch & 0xFF))

/*!
 * Enumeration of IPU logical channels. An IPU logical channel is defined as a
 * combination of an input (memory to IPU), output (IPU to memory), and/or 
 * secondary input IDMA channels and in some cases an Image Converter task. 
 * Some channels consist of only an input or output.
 */
typedef enum {
        CSI_MEM         = _MAKE_CHAN(1,0xFF,7,0xFF),    /*!< CSI raw sensor data to memory */

        CSI_PRP_ENC_MEM = _MAKE_CHAN(2,0xFF,0,0xFF),    /*!< CSI to IC Encoder PreProcessing to Memory */
        MEM_PRP_ENC_MEM = _MAKE_CHAN(3,6,0,0xFF),       /*!< Memory to IC Encoder PreProcessing to Memory */
        MEM_ROT_ENC_MEM = _MAKE_CHAN(4,10,8,0xFF),      /*!< Memory to IC Encoder Rotation to Memory */

        CSI_PRP_VF_MEM  = _MAKE_CHAN(5,0xFF,1,0xFF),    /*!< CSI to IC Viewfinder PreProcessing to Memory */
        CSI_PRP_VF_ADC  = _MAKE_CHAN(6,0xFF,1,0xFF),    /*!< CSI to IC Viewfinder PreProcessing to ADC */
        MEM_PRP_VF_MEM  = _MAKE_CHAN(7,6,1,3),          /*!< Memory to IC Viewfinder PreProcessing to Memory */
        MEM_PRP_VF_ADC  = _MAKE_CHAN(8,6,1,3),          /*!< Memory to IC Viewfinder PreProcessing to ADC */
        MEM_ROT_VF_MEM  = _MAKE_CHAN(9,11,9,0xFF),      /*!< Memory to IC Viewfinder Rotation to Memory */

        MEM_PP_MEM      = _MAKE_CHAN(10,5,2,4),         /*!< Memory to IC PostProcessing to Memory */
        MEM_ROT_PP_MEM  = _MAKE_CHAN(11,13,12,0xFF),    /*!< Memory to IC PostProcessing Rotation to Memory */
        MEM_PP_ADC      = _MAKE_CHAN(12,5,2,4),         /*!< Memory to IC PostProcessing to ADC */

        MEM_SDC_BG      = _MAKE_CHAN(14,14,0xFF,0xFF),  /*!< Memory to SDC Background plane */
        MEM_SDC_FG      = _MAKE_CHAN(15,15,0xFF,0xFF),  /*!< Memory to SDC Foreground plane */    
        MEM_SDC_MASK    = _MAKE_CHAN(16,16,0xFF,0xFF),  /*!< Memory to SDC Mask*/

        ADC_SYS1        = _MAKE_CHAN(17,18,22,20),      /*!< Memory to ADC System Channel 1 */
        ADC_SYS2        = _MAKE_CHAN(18,19,23,21),      /*!< Memory to ADC System Channel 2 */

        MEM_PF_Y_MEM    = _MAKE_CHAN(19,26,29,24  ),    /*!< Y and PF Memory to Post-filter to Y Memory */
        MEM_PF_U_MEM    = _MAKE_CHAN(20,27,30,25  ),    /*!< U and PF Memory to Post-filter to U Memory */
        MEM_PF_V_MEM    = _MAKE_CHAN(21,28,31,0xFF),    /*!< V Memory to Post-filter to V Memory */

        CHAN_NONE = 0xFFFFFFFF,
} ipu_channel_t;

/*!
 * Enumeration of types of buffers for a logical channel.
 */
typedef enum {
        IPU_INPUT_BUFFER,       /*!< Buffer for input to IPU */
        IPU_OUTPUT_BUFFER,      /*!< Buffer for output from IPU */
        IPU_SEC_INPUT_BUFFER,   /*!< Buffer for second input to IPU */
} ipu_buffer_t;

/*!
 * Enumeration of DI ports for ADC.
 */
typedef enum 
{
        DISP0,
        DISP1,
        DISP2,
        DISP3
} display_port_t;

/*!
 * Enumeration of ADC channel operation mode.
 */
typedef enum 
{
        Disable,
        WriteTemplateNonSeq,
        ReadTemplateNonSeq,
        WriteTemplateUnCon,
        ReadTemplateUnCon,
        WriteDataWithRS,
        WriteDataWoRS,
        WriteCmd 
} mcu_mode_t;


/*!
 * Enumeration of ADC channel addressing mode.
 */
typedef enum 
{
        FullWoBE,
        FullWithBE,
        XY
} display_addressing_t;


/*!
 * Union of initialization parameters for a logical channel.
 */
typedef union {
        struct {
                ulong temp;
        } csi_mem;
        struct {
                ulong in_width;
                ulong in_height;
                ulong in_pixel_fmt;
                ulong out_width;
                ulong out_height;
                ulong out_pixel_fmt;
        } csi_prp_enc_mem;
        struct {
                ulong in_width;
                ulong in_height;
                ulong in_pixel_fmt;
                ulong out_width;
                ulong out_height;
                ulong out_pixel_fmt;
        } mem_prp_enc_mem;
        struct {
                ulong in_width;
                ulong in_height;
                ulong in_pixel_fmt;
                ulong out_width;
                ulong out_height;
                ulong out_pixel_fmt;
        } mem_rot_enc_mem;
        struct {
                ulong in_width;
                ulong in_height;
                ulong in_pixel_fmt;
                ulong out_width;
                ulong out_height;
                ulong out_pixel_fmt;
                int graphics_combine_en;
		int global_alpha_en;
		int key_color_en;
        } csi_prp_vf_mem;
        struct {
                ulong in_width;
                ulong in_height;
                ulong in_pixel_fmt;
                ulong out_width;
                ulong out_height;
                ulong out_pixel_fmt;
                int graphics_combine_en;
		int global_alpha_en;
		int key_color_en;
                display_port_t disp;
                ulong out_left;
                ulong out_top;
        } csi_prp_vf_adc;
        struct {
                ulong in_width;
                ulong in_height;
                ulong in_pixel_fmt;
                ulong out_width;
                ulong out_height;
                ulong out_pixel_fmt;
                int graphics_combine_en;
		int global_alpha_en;
		int key_color_en;
        } mem_prp_vf_mem;
        struct {
                ulong temp;
        } mem_prp_vf_adc;
        struct {
                ulong temp;
        } mem_rot_vf_mem;
        struct {
                ulong in_width;
                ulong in_height;
                ulong in_pixel_fmt;
                ulong out_width;
                ulong out_height;
                ulong out_pixel_fmt;
                int graphics_combine_en;
		int global_alpha_en;
		int key_color_en;
        } mem_pp_mem;
        struct {
                ulong temp;
        } mem_rot_mem;
        struct {
                ulong in_width;
                ulong in_height;
                ulong in_pixel_fmt;
                ulong out_width;
                ulong out_height;
                ulong out_pixel_fmt;
                int graphics_combine_en;
		int global_alpha_en;
		int key_color_en;
                display_port_t disp;
                ulong out_left;
                ulong out_top;
        } mem_pp_adc;

        struct {
                pf_operation_t operation;
        } mem_pf_mem;

        struct {
                ulong temp;
        } mem_sdc_fg;
        struct {
                ulong temp;
        } mem_sdc_bg;
        struct {
                display_port_t disp;
                mcu_mode_t ch_mode;
                ulong out_left;
                ulong out_top;
        } adc_sys1;
        struct {
                display_port_t disp;
                mcu_mode_t ch_mode;
                ulong out_left;
                ulong out_top;
        } adc_sys2;
} ipu_channel_params_t;

/*!
 * Bitfield of Display Interface signal polarities.
 */
typedef struct {
    unsigned datamask_en:1;
    unsigned clksel_en:1;
    unsigned clkidle_en:1;
    unsigned data_pol:1;        // true = inverted
    unsigned clk_pol:1;         // true = rising edge
    unsigned enable_pol:1;
    unsigned Hsync_pol:1;       // true = active high
    unsigned Vsync_pol:1;
} ipu_di_signal_cfg_t;

/*!
 * Bitfield of CSI signal polarities and modes.
 */
typedef struct {
    unsigned data_width : 3;
    unsigned clk_mode   : 2;
    unsigned ext_vsync  : 1;
    unsigned Vsync_pol  : 1;
    unsigned Hsync_pol  : 1;
    unsigned pixclk_pol : 1;
    unsigned data_pol   : 1;
    unsigned sens_clksrc: 1;
} ipu_csi_signal_cfg_t;

/*!
 * Enumeration of CSI data bus widths.
 */
enum {
    IPU_CSI_DATA_WIDTH_4,
    IPU_CSI_DATA_WIDTH_8,
    IPU_CSI_DATA_WIDTH_10,
    IPU_CSI_DATA_WIDTH_16,
};

/*!
 * Enumeration of CSI clock modes.
 */
enum {
    IPU_CSI_CLK_MODE_GATED_CLK,
    IPU_CSI_CLK_MODE_NONGATED_CLK,
    IPU_CSI_CLK_MODE_CCIR656_PROGRESSIVE,
    IPU_CSI_CLK_MODE_CCIR656_INTERLACED,
};

typedef enum {
    RGB,
    YCbCr,
    YUV
} ipu_color_space_t;


/*!
 * Enumeration of ADC vertical sync mode.
 */
typedef enum
{
        VsyncNone,
        VsyncInternal,
        VsyncCSI,
        VsyncExternal
}vsync_t;

typedef enum
{
        DAT,
        CMD
}cmddata_t;

/*!
 * Enumeration of ADC display update mode.
 */
typedef enum {
        IPU_ADC_REFRESH_NONE,
        IPU_ADC_AUTO_REFRESH,
        IPU_ADC_AUTO_REFRESH_SNOOP,
        IPU_ADC_SNOOPING,
} ipu_adc_update_mode_t;

/*!
 * Enumeration of ADC display interface types (serial or parallel).
 */
enum
{
    IPU_ADC_IFC_MODE_SYS80_TYPE1,
    IPU_ADC_IFC_MODE_SYS80_TYPE2,
    IPU_ADC_IFC_MODE_SYS68K_TYPE1,
    IPU_ADC_IFC_MODE_SYS68K_TYPE2,
    IPU_ADC_IFC_MODE_3WIRE_SERIAL,
    IPU_ADC_IFC_MODE_4WIRE_SERIAL,
    IPU_ADC_IFC_MODE_5WIRE_SERIAL_CLK,
    IPU_ADC_IFC_MODE_5WIRE_SERIAL_CS,
};

enum
{
    IPU_ADC_IFC_WIDTH_8,
    IPU_ADC_IFC_WIDTH_16,
};

/*!
 * Enumeration of ADC display interface burst mode.
 */
enum
{
    IPU_ADC_BURST_WCS,
    IPU_ADC_BURST_WBLCK,
    IPU_ADC_BURST_NONE,
    IPU_ADC_BURST_SERIAL,
};

/*!
 * Enumeration of ADC display interface RW signal timing modes.
 */
enum
{
    IPU_ADC_SER_NO_RW,
    IPU_ADC_SER_RW_BEFORE_RS,
    IPU_ADC_SER_RW_AFTER_RS,
};

/*!
 * Bitfield of ADC signal polarities and modes.
 */
typedef struct
{
    unsigned data_pol:1;
    unsigned clk_pol:1;
    unsigned cs_pol:1;
    unsigned addr_pol:1;
    unsigned read_pol:1;
    unsigned write_pol:1;
    unsigned Vsync_pol:1;
    unsigned burst_pol:1;
    unsigned burst_mode:2;
    unsigned ifc_mode:3;
    unsigned ifc_width:5;
    unsigned ser_preamble_len:4;
    unsigned ser_preamble:8;
    unsigned ser_rw_mode:2;
} ipu_adc_sig_cfg_t;

/*!
 * Enumeration of ADC template commands.
 */
enum {
        RD_DATA,
        RD_ACK,
        RD_WAIT,
        WR_XADDR,
        WR_YADDR,
        WR_ADDR,
        WR_CMND,
        WR_DATA,
};

/*!
 * Enumeration of ADC template command flow control.
 */
enum {
        SINGLE_STEP,
        PAUSE,
        STOP,
};

//Define template constants
#define     ATM_ADDR_RANGE      0x20 //offset address of DISP
#define     TEMPLATE_BUF_SIZE   0x20 //size of template

#define CSI_MCLK_VF  1
#define CSI_MCLK_ENC 2
#define CSI_MCLK_RAW 4
#define CSI_MCLK_I2C 8

/* Common IPU API */
ulong ipu_init_channel(ipu_channel_t channel, ipu_channel_params_t * params );

ulong ipu_init_channel_buffer(ipu_channel_t channel, ipu_buffer_t type,
                                ulong pixel_fmt,
                                ushort width, ushort height,
                                ulong stride,
                                ipu_rotate_mode_t rot_mode,
                                void * phyaddr_0, void * phyaddr_1);

ulong ipu_select_buffer(ipu_channel_t channel, ipu_buffer_t type, ulong bufNum);
    
ulong ipu_enable_channel(ipu_channel_t channel);

/* SDC API */
ulong ipu_sdc_init_panel(ipu_panel_t panel, 
                           uchar refreshRate,
                           ushort width, ushort height,
                           ulong pixel_fmt, 
                           ushort hStartWidth, ushort hSyncWidth, ushort hEndWidth, 
                           ushort vStartWidth, ushort vSyncWidth, ushort vEndWidth,
                           ipu_di_signal_cfg_t sig);
ulong ipu_sdc_set_window_pos(ipu_channel_t channel, ushort x_pos, ushort y_pos);
ulong ipu_sdc_set_global_alpha(int enable, uchar alpha);
ulong ipu_sdc_set_color_key(ipu_channel_t channel, int enable, ulong colorKey);
ulong ipu_sdc_set_brightness(uchar value);

#endif	// __CPU_IPU_H
