// board specific defines for micro monitor
#define DEFAULT_BAUD_RATE   	38400
//------------------------------------------
// Clock Defines for CSB781
//------------------------------------------
// We are using a 33MHz Crystal as the input to the CPU.
// SYS_CLK = ((OSC*SPMF)/SYSDIV)
// SYS_CLK = ((33MHz*16)/2) = 264MHz

//#define SYS_CLK		245760000	// Used for CSB781 revision P2
#define SYS_CLK		264000000
#define CSB_CLK		(SYS_CLK/2)
#define IPS_DIV_DEF		2	// Valid IPS dividers are 2, 3, 4, and 6
#define PSC_CLK		(CSB_CLK/IPS_DIV_DEF)

#define LOOPS_PER_USEC			15

//#define BIG_ENDIAN 		// big endian, when desired
//#define LITTLE_ENDIAN 		// big endian, when desired

//-----------------------------------------------------------------------------
// MPC5121e Internal Memory Map Registers Base Address 
// All I/O are offsets.  Note that the base address can be
// programmed.  The default base address is 0xFF400000 but
// we fix it at 0xE0000000.
#define IMMRBAR_BASE_ADD		0x80000000

// For use in nbuf.c
#define INT_SRAM_ADDRESS        0x30000000
#define INT_SRAM_SIZE           0x00020000	// 128 Kbytes

//// I2C Control/Status Register
//#define I2C_REG_ADD	0xff000100
//#define I2C_REG		*(vulong *)(I2C_REG_ADD)
//// Write control bits
//#define I2C_REG_CLR_ALL 		0x00
//#define I2C_REG_SET_ALL 		0xff
//#define I2C_REG_CLR_SCL 		BIT0
//#define I2C_REG_SET_SCL 		BIT1
//#define I2C_REG_CLR_SDA			BIT2
//#define I2C_REG_SET_SDA			BIT3
//// Read Bit Locations
//#define I2C_REG_SDA_RD			BIT0
//#define I2C_REG_SCL_RD			BIT1

// LCD Defines
//
// The LCD frame buffer is fixed at 0x00200000, which is 2Mbyte from the
// beginning of SDRAM space.  Note that we access it 16-bits at a time.
//--------------------------------------------------------------------------
#define BE2LE(_x_)	((((_x_) & 0x000000ff) << 24) | (((_x_) & 0x0000ff00) << 8) | (((_x_) & 0x00ff0000) >> 8) | (((_x_) & 0xff000000) >> 24))

#define LCD_ADR_ADD				0x00200000
#define LCD_ADR(_x_)			*(vulong *)(LCD_ADR_ADD + _x_)	// Area Descriptor

#define LCD_GAMMA_ADD			0x00200040
#define LCD_GAMMA_BUF(_x_)		*(vulong *)(LCD_GAMMA_ADD + _x_)	// Gamma Buffer descriptor

#define LCD_BUF_ADD				0x00200400	
#define LCD_BUF(_x_)			*(vushort *)(LCD_BUF_ADD + _x_)	// Frame Buffer

#define USE_FONT8X8
#define LCD_GET_PIXEL_ADD(_X_, _Y_)	(((_Y_ * PIXELS_PER_ROW) + _X_)*2)

// defines for the display geometry - OSD043TN24 480x272 TFT
// (some of these defines are also used by uMon's frame buffer interface)
#define PIXELS_PER_ROW			480		//
#define PIXELS_PER_COL			272		//
#define BITS_PER_PIXEL			16		//
#define PIXFMT_IS_RGB565		1
#define FBDEV_SETSTART			fbdev_setstart
#define FRAME_BUFFER_BASE_ADDR	LCD_BUF_ADD
#define LCD_H_WIDTH 			41		// pulse width in pixels
#define LCD_H_FRONT 			2		// front porch (sync to enable)
#define LCD_H_BACK 				2		// back porch (enable to sync)
#define LCD_V_WIDTH 			10		// pulse width in lines
#define LCD_V_FRONT 			2		// front porch (sync to enable)
#define LCD_V_BACK 				2		// back porch (enable to sync)

// defines for the display geometry - OSD 800 x 480
//#define PIXELS_PER_ROW			800
//#define PIXELS_PER_COL			480
//#define BITS_PER_PIXEL			16
//#define PIXFMT_IS_RGB565		1
//#define FBDEV_SETSTART			fbdev_setstart
//#define FRAME_BUFFER_BASE_ADDR	LCD_BUF_ADD
//#define LCD_H_WIDTH 			30	// pulse width in pixels
//#define LCD_H_FRONT 			60	// front porch (sync to enable)
//#define LCD_H_BACK 				60	// back porch (enable to sync)
//#define LCD_V_WIDTH 			2	// pulse width in lines
//#define LCD_V_FRONT 			10	// front porch (sync to enable)
//#define LCD_V_BACK 				28	// back porch (enable to sync)

#define TOP						1
#define BOTTOM					(PIXELS_PER_COL-1)
#define LEFT					0
#define RIGHT					(PIXELS_PER_ROW-1)
#define CENTER_X				(PIXELS_PER_ROW/2)
#define CENTER_Y				(PIXELS_PER_COL/2)

#define ROWS_PER_SCREEN			17
#define COLS_PER_SCREEN			60

#define LCD_BG_DEF				9
#define LCD_FG_DEF				15

#define LCD_FB_SIZE(_depth_)				(((PIXELS_PER_COL * PIXELS_PER_ROW) * _depth_) / 8)

#define LCD_ROW_SIZE(_depth_)				((PIXELS_PER_ROW * _depth_) / 8)

#define LCD_GET_ADD(_row_, _col_, _depth_)	(((((_row_ * PIXELS_PER_ROW) * FONT_HEIGHT) \
											  + (_col_ * FONT_WIDTH)) \
											  * _depth_) / 8)


