//mpc5121e_ioctrl.c

#include "config.h"
#include "cpuio.h"
#include "stddefs.h"
#include "genlib.h"
#include "mpc5121e.h"

/* IO pin fields */
#define IO_PIN_FMUX(_x_)	((_x_ & 0x3) << 7)	/* pin function */
#define IO_PIN_HOLD(_x_)	((_x_ & 0x3) << 5)	/* hold time, pci only */
#define IO_PIN_PU		(1 << 4)	/* if PUE, 0=pull-down, 1=pull-up */
#define IO_PIN_PD		(0 << 4)	/* if PUE, 0=pull-down, 1=pull-up */
#define IO_PIN_PUE		(1 << 3)	/* pull up/down enable */
#define IO_PIN_PUD		(0 << 3)	/* pull up/down disable */
#define IO_PIN_STE		(1 << 2)	/* schmitt trigger enable*/
#define IO_PIN_STD		(0 << 2)	/* schmitt trigger disable*/
#define IO_PIN_DS3		(3 << 0)	/* slew rate configuration 3*/
#define IO_PIN_DS2		(2 << 0)	/* slew rate configuration 2*/
#define IO_PIN_DS1		(1 << 0)	/* slew rate configuration 1*/
#define IO_PIN_DS0		(0 << 0)	/* slew rate configuration 0*/

void ioctrl_init()
{
// Set GPIO mux correctly for either GPIO or GPTR
IO_CTRL_REG(IOCTL_GP) = 0x00000043;
//IO_CTRL_REG(IOCTL_GP) = 0x000000ff;

// Set Pin Mux for FEC signals
IO_CTRL_REG(IOCTL_PCI_AD17) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_AD16) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_AD15) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_AD14) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_AD13) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_AD12) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_AD11) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_AD10) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_AD09) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_STD | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_AD08) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_STD | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_AD07) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_AD06) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_AD05) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_AD04) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_AD03) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_AD02) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_AD01) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_AD00) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;

// Set Pin Mux for LCD Data Signals
IO_CTRL_REG(IOCTL_PCI_CBE0)	= IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_CBE1)	= IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_CBE2)	= IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_CBE3) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_GNT2) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_REQ2) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_GNT1) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_REQ1) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_GNT0) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_REQ0) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_RST_OUT) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_FRAME) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_IRDY) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_TRDY) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_STOP) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_PAR)  = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_PERR) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;
IO_CTRL_REG(IOCTL_PCI_SERR) = IO_PIN_FMUX(0x2) | IO_PIN_HOLD(0x0) | IO_PIN_DS0;

// Set Pin Mux for LCD Backlight Control Signal
IO_CTRL_REG(IOCTL_PSC6_4) = IO_PIN_FMUX(0x3) | IO_PIN_DS2;

// Set Pin Mux for LCD Control Signals
IO_CTRL_REG(IOCTL_PSC11_4) = IO_PIN_FMUX(0x2) | IO_PIN_DS2;
IO_CTRL_REG(IOCTL_SPDIF_TXCLK) = IO_PIN_FMUX(0x2) | IO_PIN_STD | IO_PIN_DS2;
IO_CTRL_REG(IOCTL_SPDIF_TX) = IO_PIN_FMUX(0x2) | IO_PIN_DS2;
IO_CTRL_REG(IOCTL_SPDIF_RX) = IO_PIN_FMUX(0x2) | IO_PIN_DS2;

// Initialization I2C_0 as GPIO for bit-bang mode, SCL = GPIO7, SDA = GPIO1
IO_CTRL_REG(IOCTL_I2C0_SCL) = IO_PIN_FMUX(0x3) | IO_PIN_STD | IO_PIN_DS2;
IO_CTRL_REG(IOCTL_I2C0_SDA) = IO_PIN_FMUX(0x3) | IO_PIN_STD | IO_PIN_DS2;
}
