/* smsc911x_.h:
 * 	Header file for the ethernet driver SMSC LAN911x.
 *
 *	General notice:
 *	This code is part of a boot-monitor package developed as a generic base
 *	platform for embedded system designs.  As such, it is likely to be
 *	distributed to various projects beyond the control of the original
 *	author.  Please notify the author of any enhancements made or bugs found
 *	so that all may benefit from the changes.  In addition, notification back
 *	to the author will allow the new user to pick up changes that may have
 *	been made by other users after this version of the code was distributed.*
 *
 *	Author:	Torsten Tetz, Bosch Security Systems
 */

/* Name used for printfs in the code. */
#define NAME_OF_NETDEVICE "SMSC911x"

/* Sending buffer size. */
#define TX_FIFO_LOW_THRESHOLD	(1600)

/* Public interface. */
void smsc911x_reset(void);
void smsc911x_PHY_reset(void);
int smsc911x_init(void);
int smsc911x_send(unsigned long *packet,int length);
unsigned long smsc911x_package_arrived(void);
int smsc911x_receive(unsigned long *dataBuffer,unsigned long length);
int smsc911x_verify_chip_ID(void);
void smsc911x_disablePromiscuousReception(void);
void smsc911x_enablePromiscuousReception(void);

/* Private functions. */
void smsc911x_start_elapsed_timer(struct elapsed_tmr *tmr,int milliseconds);
int smsc911x_msec_elapsed(struct elapsed_tmr *tmr);
void smsc911x_set_MIIPHY_reg(unsigned char phyAddress,unsigned char miiRegister,unsigned short data);
unsigned short smsc911x_get_MIIPHY_reg(unsigned char phyAddress,unsigned char miiRegister);
unsigned char smsc911x_MAC_NOT_BUSY(void);
void smsc911x_set_MAC_CSR(unsigned char registerSymbol,unsigned long data);
unsigned long smsc911x_get_MAC_CSR(unsigned char registerSymbol);
void smsc911x_PHY_configure(void);
int smsc911x_set_mac_address(unsigned char *BinEnetAddr);
void smsc911x_enable(void);
void write_data_to_TX_FIFO_DATA(unsigned long *dataBuf,unsigned long ulCount);

/* Macros for setting and getting network registers. */
#define SMSC911X_GET_REG(reg)	\
	((*(volatile unsigned long *)(DRIVER_SMSC911X_BASE + reg)))
#define SMSC911X_SET_REG(reg,value)	\
	((*(volatile unsigned long *)(DRIVER_SMSC911X_BASE + reg)) = (value))

#define SMSC911X_GET_REG_BITS(reg,bits)	\
	((*(volatile unsigned long *)(DRIVER_SMSC911X_BASE + reg)) & (bits))

#define SMSC911X_CLR_REG_BITS(reg,bits)	\
	((*(volatile unsigned long *)(DRIVER_SMSC911X_BASE + reg)) &= (~bits))
#define SMSC911X_SET_REG_BITS(reg,bits)	\
	((*(volatile unsigned long *)(DRIVER_SMSC911X_BASE + reg)) |= (bits))

/* Below are the register offsets and bit definitions.
 * for the LAN911x memory space.
 */

/* RX Data FIFO Port */
#define		RX_DATA_FIFO		(0x00UL)

/* TX Data FIFO Port */
#define 	TX_DATA_FIFO		(0x20UL)
#define		TX_CMD_A_FIRST_SEG_	(0x00002000UL)
#define		TX_CMD_A_LAST_SEG_	(0x00001000UL)

/* RX Status FIFO*/
#define		RX_STATUS_FIFO		(0x40UL)
#define		RX_STS_PAKTLENGTH_	(0x3FFF0000UL)
#define		RX_STS_ES_		(0x00008000UL)
#define		RX_STS_RUNT_FRAME_	(0x00000800UL)
#define		RX_STS_FRAME_TO_LONG_	(0x00000080UL)
#define		RX_STS_COLLISION_SEEN_	(0x00000040UL)
#define		RX_STS_CRC_ERROR_	(0x00000002UL)

/* RX Status FIFO PEEK */
#define		RX_STATUS_FIFO_PEEK	(0x44UL)

/* TX Status FIFO */
#define		TX_STATUS_FIFO		(0x48UL)
#define		TX_STS_TAG_		(0xFFFF0000)
#define		TX_STS_ES_		(0x00008000)
#define		TX_STS_LOC_		(0x00000800)
#define		TX_STS_NO_CARR_		(0x00000400)
#define		TX_STS_LATE_COLL_	(0x00000200)
#define		TX_STS_MANY_COLL_	(0x00000100)
#define		TX_STS_COLL_CNT_	(0x00000078)
#define		TX_STS_MANY_DEFER_	(0x00000004)
#define		TX_STS_UNDERRUN_	(0x00000002)
#define		TX_STS_DEFERRED_	(0x00000001)

/* Chip ID and Revision */
#define		ID_REV			(0x50UL)

/* Byte Order Test Register */
#define		BYTE_TEST		(0x64UL)

/* Interrupt Enable Register */
#define		INT_EN			(0x5CUL)
#define		INT_EN_SW_INT_EN_	(0x80000000UL)	/* R/W */
#define		INT_EN_TXSTOP_INT_EN_	(0x02000000UL)	/* R/W */
#define		INT_EN_RXSTOP_INT_EN_	(0x01000000UL)	/* R/W */
#define		INT_EN_RXDFH_INT_EN_	(0x00800000UL)	/* R/W */
#define		INT_EN_TIOC_INT_EN_	(0x00200000UL)	/* R/W */
#define		INT_EN_RXD_INT_EN_	(0x00100000UL)	/* R/W */
#define		INT_EN_GPT_INT_EN_	(0x00080000UL)	/* R/W */
#define		INT_EN_PHY_INT_EN_	(0x00040000UL)	/* R/W */
#define		INT_EN_PME_INT_EN_	(0x00020000UL)	/* R/W */
#define		INT_EN_TXSO_EN_		(0x00010000UL)	/* R/W */
#define		INT_EN_RWT_EN_		(0x00008000UL)	/* R/W */
#define		INT_EN_RXE_EN_		(0x00004000UL)	/* R/W */
#define		INT_EN_TXE_EN_		(0x00002000UL)	/* R/W */
#define		INT_EN_TDFU_EN_		(0x00000800UL)	/* R/W */
#define		INT_EN_TDFO_EN_		(0x00000400UL)	/* R/W */
#define		INT_EN_TDFA_EN_		(0x00000200UL)	/* R/W */
#define		INT_EN_TSFF_EN_		(0x00000100UL)	/* R/W */
#define		INT_EN_TSFL_EN_		(0x00000080UL)	/* R/W */
#define		INT_EN_RXDF_EN_		(0x00000040UL)	/* R/W */
#define		INT_EN_RDFL_EN_		(0x00000020UL)  /* R/W */
#define		INT_EN_RSFF_EN_		(0x00000010UL)	/* R/W */
#define		INT_EN_RSFL_EN_		(0x00000008UL)	/* R/W */
#define		INT_EN_GPIO2_INT_	(0x00000004UL)	/* R/W */
#define		INT_EN_GPIO1_INT_	(0x00000002UL)	/* R/W */
#define		INT_EN_GPIO0_INT_	(0x00000001UL)	/* R/W */

/* Receive Configuration */
#define 	RX_CFG			(0x6CUL)

/* Transmit Configuration */
#define 	TX_CFG			(0x70UL)
#define		TX_CFG_TX_ON_		(0x00000002UL)	/* R/W */

/* Harware Configuration Register */
#define		HW_CFG			(0x74UL)
#define		HW_CFG_SF_		(0x00100000UL)	/* R/W */
#define		HW_CFG_TX_FIF_SZ_	(0x000F0000UL)	/* R/W */
#define		HW_CFG_SRST_		(0x00000001UL)	/* Self clearing */

/* Receive FIFO Information */
#define		RX_FIFO_INF		(0x7CUL)
#define		RX_FIFO_INF_RXSUSED_	(0x00FF0000UL)	/* RO */
#define		RX_FIFO_INF_RXDUSED_	(0x0000FFFFUL)	/* RO */

/* Transmit FIFO Information */
#define		TX_FIFO_INF		(0x80UL)
#define		TX_FIFO_INF_TSUSED_	(0x00FF0000UL)  /* RO */
#define		TX_FIFO_INF_TDFREE_	(0x0000FFFFUL)	/* RO */

/* Power Management Control Register */
#define		PMT_CTRL		(0x84UL)
#define		PMT_CTRL_READY_		(0x00000001UL)	/* RO */
#define		PMT_CTRL_PHY_RST_	(0x00000400UL)	/* Self clearing */

/* Gereral Purpose IO Configuration Register */
#define		GPIO_CFG		(0x88UL)
#define		GPIO_CFG_LED3_EN_	(0x40000000UL)	/* R/W */
#define		GPIO_CFG_LED2_EN_	(0x20000000UL)	/* R/W */
#define		GPIO_CFG_LED1_EN_	(0x10000000UL)	/* R/W */
#define		GPIO_CFG_GPIOBUF2_	(0x00040000UL)	/* R/W */
#define		GPIO_CFG_GPIOBUF1_	(0x00020000UL)	/* R/W */
#define		GPIO_CFG_GPIOBUF0_	(0x00010000UL)	/* R/W */

/* General Purpose Timer Configuration Register. */
#define 	GPT_CFG			(0x8CUL)
#define		GPT_CFG_TIMER_EN_	(0x20000000UL)	/* R/W */

/* Automatic Flow Control Configuration Register */
#define		AFC_CFG			(0xACUL)
#define		AFC_CFG_AFC_HI_		(0x00FF0000UL)	/* R/W */
#define		AFC_CFG_AFC_LO_		(0x0000FF00UL)	/* R/W */
#define		AFC_CFG_BACK_DUR_	(0x000000F0UL)	/* R/W */
#define		AFC_CFG_FCMULT_		(0x00000008UL)	/* R/W */
#define		AFC_CFG_FCBRD_		(0x00000004UL)	/* R/W */
#define		AFC_CFG_FCADD_		(0x00000002UL)	/* R/W */
#define		AFC_CFG_FCANY_		(0x00000001UL)	/* R/W */

/* EEPROM Command Register */
#define		E2P_CMD			(0xB0UL)
#define		E2P_CMD_EPC_BUSY_	(0x80000000UL)	/* Self clearing */

/* MAC Control and Status Register Setting Registers. */
#define 	MAC_CSR_DATA		(0xA8UL)
#define 	MAC_CSR_CMD		(0xA4UL)
#define		MAC_CSR_CMD_CSR_BUSY_	(0x80000000UL)	/* Self Clearing */
#define		MAC_CSR_CMD_R_NOT_W_	(0x40000000UL)	/* R/W */


/* MAC Control and Status Registers:
 * The following registers can only be accesed indirectly trough 
 * MAC_CSR_DATA and MAC_CSR_CMD! 
 */

/* MAC Control Register */
#define 	MAC_CR			(0x01UL)	/* R/W */
#define 	MAC_CR_HBDIS_		(0x10000000UL)
#define		MAC_CR_PRMS_		(0x00040000UL)
#define		MAC_CR_TXEN_		(0x00000008UL)
#define		MAC_CR_RXEN_		(0x00000004UL)

/* MAC Address High */
#define 	ADDRH			(0x02UL)	/* R/W mask 0x0000FFFFUL */

/* MAC Address Low */
#define 	ADDRL			(0x03UL)	/* R/W mask 0xFFFFFFFFUL */

/* Flow Control Register */
#define 	FLOW			(0x08UL)	/* R/W */
#define		FLOW_FCBSY_		(0x00000001UL)	
#define 	FLOW_FCPT_		(0xFFFF0000UL)
#define 	FLOW_FCEN_		(0x00000002UL)

/* MII Access Register */
#define 	MII_ACC			(0x06UL)	/* R/W */
#define		MII_ACC_MII_WRITE_	(0x00000002UL)
#define		MII_ACC_MII_BUSY_	(0x00000001UL)

/* MII Data Register */
#define 	MII_DATA		(0x07UL)	/* R/W mask 0x0000FFFFUL */


/* The following PHY registers can only be accesed indirectly trough 
 * MAC_CSR via MII_ACC and MII_DATA
 */

/* Basic Control Register */
#define 	PHY_BCR				((unsigned long)0U)
#define 	PHY_BCR_AUTO_NEG_ENABLE_	((unsigned short)0x1000U)
#define 	PHY_BCR_RESTART_AUTO_NEG_	((unsigned short)0x0200U)

/* Basic Status Register */
#define 	PHY_BSR			((unsigned long)1U)
#define 	PHY_BSR_LINK_STATUS_	((unsigned short)0x0004U)

/* Auto-negotiation Advertisement */
#define 	PHY_ANEG_ADV    	((unsigned long)4U)
#define		PHY_SELECTOR_FIELD_	((unsigned short)0x001)	/* [00001] = IEEE 802.3 */
#define 	PHY_ANEG_ADV_10H_	((unsigned short)0x020)
#define 	PHY_ANEG_ADV_10F_	((unsigned short)0x040)
#define 	PHY_ANEG_ADV_100H_	((unsigned short)0x080)
#define 	PHY_ANEG_ADV_100F_	((unsigned short)0x100)

