/* timerdev.c:
 *	Here is the code for using the SH7720 board based timer unit.
 *
 *	The target_timer() function:
 *	(Used by various uMon code, when available.) 
 *	This version here uses the Timer Unit(TMU, channel 0) of the MPR2.
 *	See target/common/timestuff.c to learn more about the use of a 
 *	board based timer in uMon.
 *
 *	General notice:
 *	This code is part of a boot-monitor package developed as a generic base
 *	platform for embedded system designs.  As such, it is likely to be
 *	distributed to various projects beyond the control of the original
 *	author.  Please notify the author of any enhancements made or bugs found
 *	so that all may benefit from the changes.  In addition, notification back
 *	to the author will allow the new user to pick up changes that may have
 *	been made by other users after this version of the code was distributed.
 *
 *	Author:	Torsten Tetz, Bosch Security Systems
 */
#include "config.h"

#if INCLUDE_HWTMR
#include "timer.h"
#include "cpuio.h"
#include "timerdev.h"

/* init_timer():
 * Start the "Timer Unit"(TMU, channel 0) for later use
 * of target timer.
 */
void 
init_timer0(void)
{

	/* Disable interrupt due to UNF (TUNI).
	 * Count on P0 * 1/4.
	 */
	*TMU_TCR_0 = (*TMU_TCR_0 & ~(TMU_TCR_UNIE | TMU_TCR_TPSC2 | TMU_TCR_TPSC1 | TMU_TCR_TPSC0));

	/* Set the value to be set in TCNT when TCNT underflows. */
	*TMU_TCOR_0 = 0xFFFFFFFF;

	/* Initialize counter with the correct start value. */
	*TMU_TCNT_0 = *TMU_TCOR_0;

	/* Start Timer 1 */
	*TMU_TSTR = (*TMU_TSTR | TMU_TSTR_STR0); 
}

/* target_timer():
 * Returns an unsigned long value that is the content of the polled 
 * hardware timer. Since the common code expects the hardware timer to count up
 * instead of down like the hardware counter here, we need to modify the value.
 *
 * Uses "Timer Unit"(TMU, channel 0) of MPR2.
 */
unsigned long 
target_timer(void)
{
	unsigned long upcount = 0xFFFFFFFF - *TMU_TCNT_0;
	return(upcount);
}

#endif	/* INCLUDE_HWTMR */
